@@ -1,6 +1,6 @@
 /******************************************************************************
  *  Wormux is a convivial mass murder game.
- *  Copyright (C) 2001-2008 Wormux Team.
+ *  Copyright (C) 2001-2010 Wormux Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,9 +21,8 @@
 
 #include <sstream>
 #include <iostream>
-#include "character/body.h"
+#include <WORMUX_random.h>
 #include "character/character.h"
-#include "character/move.h"
 #include "character/damage_stats.h"
 #include "game/config.h"
 #include "game/game_mode.h"
@@ -43,7 +42,6 @@
 #include "team/custom_team.h"
 #include "team/macro.h"
 #include "tool/math_tools.h"
-#include "tool/random.h"
 #include "tool/string_tools.h"
 #include "weapon/explosion.h"
 
@@ -57,7 +55,7 @@ const uint HAUT_FONT_MIX = 13;
 // Space between the name, the skin and the energy bar
 const uint ESPACE = 3; // pixels
 const uint do_nothing_timeout = 5000;
-const double MIN_SPEED_TO_FLY = 15.0;
+const Double MIN_SPEED_TO_FLY = 15.0;
 
 // Pause for the animation
 #ifdef DEBUG
@@ -80,24 +78,42 @@ const uint LARG_ENERGIE = 40;
 const uint HAUT_ENERGIE = 6;
 
 // Delta angle used to move the crosshair
-const double DELTA_CROSSHAIR = 0.035; /* ~1 degree */
+const Double DELTA_CROSSHAIR = 0.035; /* ~1 degree */
+
+// Pause between changing direction
+const uint PAUSE_CHG_DIRECTION = 80; // ms
+
+// Max climbing height walking
+const int MAX_CLIMBING_HEIGHT=30;
+
+// Max height for which we do not need to call the Physical Engine with gravity features
+const int MAX_FALLING_HEIGHT=20;
 
 /* FIXME This methode is really strange, all this should probably be done in
  * constructor of Body...*/
-void Character::SetBody(Body* char_body)
+void Character::SetBody(Body * char_body)
 {
   body = char_body;
   body->SetOwner(this);
   SetClothe("normal");
   SetMovement("breathe");
 
-  SetDirection(RandomLocal().GetBool() ? DIRECTION_LEFT : DIRECTION_RIGHT);
-  body->SetFrame(RandomLocal().GetLong(0, body->GetFrameCount() - 1));
+  MSG_DEBUG("random.get", "Character::SetBody(...) direction");
+  SetDirection(RandomSync().GetBool() ? DIRECTION_LEFT : DIRECTION_RIGHT);
+  MSG_DEBUG("random.get", "Character::SetBody(...) body frame");
+  body->SetFrame(RandomSync().GetLong(0, body->GetFrameCount() - 1));
   SetSize(body->GetSize());
 }
 
+static uint GetRandomAnimationTimeValue()
+{
+  MSG_DEBUG("random.get", "Character::SetBody(...) body frame");
+  return Time::GetInstance()->Read() + RandomSync().GetLong(ANIM_PAUSE_MIN,ANIM_PAUSE_MAX);
+}
+
 Character::Character (Team& my_team, const std::string &name, Body *char_body) :
   PhysicalObj("character"),
+  MovableByUser(),
   character_name(name),
   m_team(my_team),
   step_sound_played(true),
@@ -108,24 +124,24 @@ Character::Character (Team& my_team, const std::string &name, Body *char_body) :
   disease_damage_per_turn(0),
   disease_duration(0),
   damage_stats(new DamageStatistics(*this)),
-  energy_bar(),
+  energy_bar(NULL),
   survivals(0),
   name_text(NULL),
   rl_motion_pause(0),
   do_nothing_time(0),
   walking_time(0),
-  animation_time(Time::GetInstance()->Read() + RandomLocal().GetLong(ANIM_PAUSE_MIN,ANIM_PAUSE_MAX)),
+  animation_time(GetRandomAnimationTimeValue()),
   lost_energy(0),
   hidden(false),
   channel_step(-1),
   particle_engine(new ParticleEngine(500)),
   is_playing(false),
+  last_direction_change(0),
   previous_strength(0),
   body(NULL)
 {
-
   m_is_character = true;
-  SetCollisionModel(false, true, true);
+  SetCollisionModel(true, true, true);
   /* body stuff */
   ASSERT(char_body);
   SetBody(char_body);
@@ -141,19 +157,24 @@ Character::Character (Team& my_team, const std::string &name, Body *char_body) :
 
   // Energy
   m_energy = GameMode::GetInstance()->character.init_energy;
-  energy_bar.InitVal (GameMode::GetInstance()->character.init_energy,
-                      0,
-                      GameMode::GetInstance()->character.init_energy);
-  energy_bar.InitPos (0,0, LARG_ENERGIE, HAUT_ENERGIE);
-  energy_bar.SetBorderColor( black_color );
-  energy_bar.SetBackgroundColor( gray_color );
+
+  energy_bar = new EnergyBar(0, 0, 
+                             LARG_ENERGIE, HAUT_ENERGIE,
+                             GameMode::GetInstance()->character.init_energy,
+                             0,
+                             GameMode::GetInstance()->character.init_energy);
+
+  energy_bar->SetBorderColor(black_color);
+  energy_bar->SetBackgroundColor(gray_color);
 
   SetEnergy(GameMode::GetInstance()->character.init_energy);
+
   MSG_DEBUG("character", "Load character %s", character_name.c_str());
 }
 
 Character::Character (const Character& acharacter) :
   PhysicalObj(acharacter),
+  MovableByUser(),
   character_name(acharacter.character_name),
   m_team(acharacter.m_team),
   step_sound_played(acharacter.step_sound_played),
@@ -164,7 +185,7 @@ Character::Character (const Character& acharacter) :
   disease_damage_per_turn(acharacter.disease_damage_per_turn),
   disease_duration(acharacter.disease_duration),
   damage_stats(new DamageStatistics(*acharacter.damage_stats, *this)),
-  energy_bar(acharacter.energy_bar),
+  energy_bar(NULL),
   survivals(acharacter.survivals),
   name_text(NULL),
   rl_motion_pause(acharacter.rl_motion_pause),
@@ -176,33 +197,56 @@ Character::Character (const Character& acharacter) :
   channel_step(acharacter.channel_step),
   particle_engine(new ParticleEngine(250)),
   is_playing(acharacter.is_playing),
+  last_direction_change(0),
   previous_strength(acharacter.previous_strength),
   body(NULL)
 {
-  if (acharacter.body)
-    SetBody(new Body(*acharacter.body));
-  if(acharacter.name_text)
+  energy_bar = new EnergyBar(acharacter.energy_bar->GetX(), 
+                             acharacter.energy_bar->GetY(),
+                             acharacter.energy_bar->GetWidth(), 
+                             acharacter.energy_bar->GetHeight(),
+                             acharacter.energy_bar->GetVal(),
+                             acharacter.energy_bar->GetMinVal(),
+                             acharacter.energy_bar->GetMaxVal());
+  energy_bar->SetBorderColor(black_color);
+  energy_bar->SetBackgroundColor(gray_color);
+  SetEnergy(GameMode::GetInstance()->character.init_energy);
+ 
+  if (acharacter.body) {
+    Body * newBody = new Body(*acharacter.body);
+    SetBody(newBody);
+  }
+
+  if (acharacter.name_text) {
     name_text = new Text(*acharacter.name_text);
+  }
 }
 
 Character::~Character()
 {
   MSG_DEBUG("character", "Unload character %s", character_name.c_str());
-  if(body)
+  if (body) {
     delete body;
-  if(name_text)
+  }
+  if (name_text) {
     delete name_text;
-  if(particle_engine)
+  }
+  if (particle_engine) {
     delete particle_engine;
-  body          = NULL;
-  name_text     = NULL;
+  }
+  if (NULL != energy_bar) {
+    delete energy_bar;
+  }
+  body            = NULL;
+  name_text       = NULL;
   particle_engine = NULL;
+  energy_bar      = NULL;
 }
 
 void Character::SignalDrowning()
 {
   // Follow character
-  Camera::GetInstance()->FollowObject(this, true);
+  Camera::GetInstance()->FollowObject(this);
 
   // Set energy
   SetEnergy(0);
@@ -223,7 +267,7 @@ void Character::SignalGhostState (bool was_dead)
   if (!was_dead) Game::GetInstance()->SignalCharacterDeath (this);
 }
 
-void Character::SetDirection (BodyDirection_t nv_direction)
+void Character::SetDirection (LRDirection nv_direction)
 {
   body->SetDirection(nv_direction);
   uint l,r,t,b;
@@ -237,19 +281,18 @@ void Character::DrawEnergyBar(int dy) const
   if( IsDead() )
         return;
 
-  energy_bar.DrawXY( Point2i( GetCenterX() - energy_bar.GetWidth() / 2, GetY() + dy)
+  energy_bar->DrawXY( Point2i( GetCenterX() - energy_bar->GetWidth() / 2, GetY() + dy)
                      - Camera::GetInstance()->GetPosition() );
 }
 
-void Character::DrawName (int dy) const
+void Character::DrawName(int dy) const
 {
-  if(IsDead()) return;
+  if (IsDead()) return;
 
-  const int x =  GetCenterX();
-  const int y = GetY()+dy;
+  const int x = GetCenterX();
+  const int y = GetY() + dy;
 
-  if (Config::GetInstance()->GetDisplayNameCharacter())
-  {
+  if (Config::GetInstance()->GetDisplayNameCharacter()) {
     name_text->DrawCenterTopOnMap(Point2i(x,y));
   }
 
@@ -311,7 +354,7 @@ void Character::SetEnergy(int new_energy)
   // Change energy
   m_energy = InRange_Long((int)new_energy, 0,
                      GameMode::GetInstance()->character.max_energy);
-  energy_bar.Actu(m_energy);
+  energy_bar->Actu(m_energy);
 
   // Dead character ?
   if (GetEnergy() <= 0) Die();
@@ -333,7 +376,10 @@ void Character::Die()
     body->SetRotation(0.0);
     SetClothe("dead");
     SetMovement("breathe");
-    SetCollisionModel(false, false, false);
+    SetCollisionModel(true, false, false);
+
+    ud_move_intentions.clear();
+    lr_move_intentions.clear();
 
     if(death_explosion)
       ApplyExplosion(GetCenter(), GameMode::GetInstance()->death_explosion_cfg);
@@ -344,6 +390,7 @@ void Character::Die()
   }
 
   damage_stats->SetDeathTime(Time::GetInstance()->Read());
+  Camera::GetInstance()->StopFollowingObj(this);
 }
 
 void Character::Draw()
@@ -367,47 +414,6 @@ void Character::Draw()
       || IsDead())
     draw_loosing_energy = false;
 
-  if (Game::GetInstance()->ReadState() == Game::END_TURN && body->IsWalking())
-    body->ResetWalk();
-
-  if (Time::GetInstance()->Read() > animation_time && !IsActiveCharacter() && !IsDead()
-      && body->GetMovement().substr(0,9) != "animation"
-      &&  body->GetClothe().substr(0,9) != "animation")
-  {
-    body->PlayAnimation();
-    animation_time = Time::GetInstance()->Read() + body->GetMovementDuration() + RandomLocal().GetLong(ANIM_PAUSE_MIN,ANIM_PAUSE_MAX);
-  }
-
-  // Stop the animation or the black skin if we are playing
-  if (IsActiveCharacter()
-      && Game::GetInstance()->ReadState() == Game::PLAYING
-      && (body->GetMovement().substr(0,9) == "animation"
-	  || body->GetClothe().substr(0,9) == "animation"
-	  || body->GetClothe() == "black"))
-  {
-    SetClothe("normal");
-    SetMovement("breathe");
-  }
-
-  // Stop flying if we don't go fast enough
-  double n, a;
-  GetSpeed(n, a);
-  if (body->GetMovement() == "fly" && n < MIN_SPEED_TO_FLY)
-    SetMovement("breathe");
-
-
-  // Refresh the body (needed to determine if "weapon-*-begin-shoot" is finnished)
-  body->Build();
-
-  if (prepare_shoot)
-  {
-    if (body->GetMovement() != "weapon-" + ActiveTeam().GetWeapon().GetID() + "-begin-shoot")
-    {
-      // if the movement is finnished, shoot !
-      DoShoot();
-      prepare_shoot = false;
-    }
-  }
 
   Point2i pos = GetPosition();
   body->Draw(pos);
@@ -441,9 +447,8 @@ void Character::Draw()
     std::ostringstream ss;
     ss << lost_energy;
     dy -= HAUT_FONT_MIX;
-    (*Font::GetInstance(Font::FONT_SMALL)).WriteCenterTop (
-        GetPosition() - Camera::GetInstance()->GetPosition() + Point2i( GetWidth()/2, dy),
-        ss.str(), white_color);
+	Text text(ss.str());
+	text.DrawCenterTop(GetPosition() - Camera::GetInstance()->GetPosition() + Point2i( GetWidth()/2, dy));
   }
 
 #ifdef DEBUG
@@ -451,7 +456,7 @@ void Character::Draw()
   if (IsLOGGING("body"))
   {
     dy -= HAUT_FONT_MIX;
-    std::string txt = body->GetClothe() + " " + body->GetMovement();
+    std::string txt = body->GetClothe() + " " + body->GetMovement() + " " + body->GetFrameLoop();
     Text skin_text(txt);
     skin_text.DrawCenterTopOnMap(Point2i(GetX(), GetY() - dy));
   }
@@ -468,14 +473,14 @@ void Character::Draw()
 #endif
 }
 
-void Character::Jump(double strength, double angle /*in radian */)
+void Character::Jump(Double strength, Double angle /*in radian */)
 {
-  Camera::GetInstance()->FollowObject(this, true);
+  Camera::GetInstance()->FollowObject(this);
 
   UpdateLastMovingTime();
   walking_time = Time::GetInstance()->Read();
 
-  if (!CanJump() && ActiveTeam().IsLocal()) return;
+  if (!CanJump()) return;
 
   SetMovement("jump");
 
@@ -504,7 +509,7 @@ void Character::BackJump()
 {
   MSG_DEBUG("character", "BackJump");
   back_jumping = true;
-  JukeBox::GetInstance()->Play (ActiveTeam().GetSoundProfile(), "jump");
+  JukeBox::GetInstance()->Play (ActiveTeam().GetSoundProfile(), "back_jump");
   Jump(GameMode::GetInstance()->character.back_jump_strength,
        GameMode::GetInstance()->character.back_jump_angle);
 }
@@ -527,13 +532,14 @@ void Character::PrepareShoot()
 
 void Character::DoShoot()
 {
-  if (Game::GetInstance()->ReadState() != Game::PLAYING)
+  if (Game::GetInstance()->ReadState() != Game::PLAYING) {
+    MSG_DEBUG("weapon.shoot", "DoShoot cancelled! time: %u", Time::GetInstance()->Read());
     return; // hack related to bugs 8656 and 9462
+  }
 
-  MSG_DEBUG("weapon.shoot", "-> begin");
+  MSG_DEBUG("weapon.shoot", "-> begin at time %u", Time::GetInstance()->Read());
   SetMovementOnce("weapon-" + ActiveTeam().GetWeapon().GetID() + "-end-shoot");
   body->Build(); // Refresh the body
-  body->UpdateWeaponPosition(GetPosition());
   damage_stats->OneMoreShot();
   ActiveTeam().AccessWeapon().Shoot();
   MSG_DEBUG("weapon.shoot", "<- end");
@@ -544,8 +550,19 @@ void Character::UpdateLastMovingTime()
   do_nothing_time = Time::GetInstance()->Read();
 }
 
+bool Character::HasGroundUnderFeets() const
+{
+  return IsImmobile() && !IsFalling();
+}
+
 void Character::Refresh()
 {
+  StartOrStopWalkingIfNecessary();
+  if (IsWalking())
+    MakeSteps();
+
+  UpdateFiringAngle();
+
   if (IsGhost()) return;
 
   UpdatePosition();
@@ -556,7 +573,12 @@ void Character::Refresh()
 
   // center on character who is falling
   if (FootsInVacuum()) {
-    Camera::GetInstance()->FollowObject(this, true);
+    bool closely = false;
+    if (IsActiveCharacter() &&
+	(ActiveTeam().GetWeaponType() == Weapon::WEAPON_JETPACK
+	 || ActiveTeam().GetWeaponType() == Weapon::WEAPON_PARACHUTE))
+      closely = true;
+    Camera::GetInstance()->FollowObject(this, closely);
   }
 
   if (IsDiseased())
@@ -565,7 +587,7 @@ void Character::Refresh()
     if (GetDirection() == DIRECTION_LEFT)
       bubble_pos.x += GetWidth();
     particle_engine->AddPeriodic(bubble_pos, particle_ILL_BUBBLE, false,
-                              - M_PI_2 - (float)GetDirection() * M_PI_4, 20.0);
+                              - HALF_PI - (Double)GetDirection() * QUARTER_PI, 20.0);
   }
 
   if (IsActiveCharacter() && Game::GetInstance()->ReadState() == Game::PLAYING)
@@ -604,14 +626,73 @@ void Character::Refresh()
   if (back_jumping)
   {
     ASSERT(&ActiveCharacter() == this);
-    double rotation;
-    static double speed_init = GameMode::GetInstance()->character.back_jump_strength *
+    Double rotation;
+    static Double speed_init = GameMode::GetInstance()->character.back_jump_strength *
        sin(GameMode::GetInstance()->character.back_jump_angle);
 
     Point2d speed = GetSpeedXY();
-    rotation = M_PI * speed.y / speed_init;
+    rotation = PI * speed.y / speed_init;
     body->SetRotation(rotation);
   }
+
+  if (Game::GetInstance()->ReadState() == Game::END_TURN && body->IsWalking())
+    body->StopWalking();
+
+  if (Time::GetInstance()->Read() > animation_time && !IsActiveCharacter() && !IsDead()
+      && body->GetMovement().substr(0,9) != "animation"
+      &&  body->GetClothe().substr(0,9) != "animation")
+  {
+    body->PlayAnimation();
+    MSG_DEBUG("random.get", "Character::Refresh()");
+    animation_time = Time::GetInstance()->Read() + body->GetMovementDuration() + RandomSync().GetLong(ANIM_PAUSE_MIN,ANIM_PAUSE_MAX);
+  }
+
+  // Stop the animation or the black skin if we are playing
+  if (IsActiveCharacter()
+      && Game::GetInstance()->ReadState() == Game::PLAYING
+      && (body->GetMovement().substr(0,9) == "animation"
+	  || body->GetClothe().substr(0,9) == "animation"
+	  || body->GetClothe() == "black"))
+  {
+    SetClothe("normal");
+    SetMovement("breathe");
+  }
+
+  // Stop flying if we don't go fast enough
+  Double n, a;
+  GetSpeed(n, a);
+  if (body->GetMovement() == "fly" && n < MIN_SPEED_TO_FLY)
+    SetMovement("breathe");
+
+
+  // Refresh the body (needed to determine if "weapon-*-begin-shoot" is finnished)
+  body->Build();
+
+  if (prepare_shoot)
+  {
+    if (body->GetMovement() != "weapon-" + ActiveTeam().GetWeapon().GetID() + "-begin-shoot")
+    {
+      // if the movement is finnished, shoot !
+      DoShoot();
+      prepare_shoot = false;
+    }
+  }
+}
+
+void Character::UpdateFiringAngle()
+{
+  bool can_change = IsImmobile() && !GetTeam().AccessWeapon().IsPreventingLRMovement();
+  const UDMoveIntention * ud_move_intention = GetLastUDMoveIntention();
+  if (can_change && ud_move_intention) {
+    UpdateLastMovingTime();
+    CharacterCursor::GetInstance()->Hide();
+    Double delta = DELTA_CROSSHAIR;
+    if (ud_move_intention->IsToDoItSlowly())
+      delta /= 10.0;
+    if (ud_move_intention->GetDirection() == DIRECTION_UP)
+      delta = -delta;
+    AddFiringAngle(delta);
+  }
 }
 
 // Prepare a new turn
@@ -622,39 +703,8 @@ void Character::PrepareTurn()
   rl_motion_pause = Time::GetInstance()->Read();
 }
 
-bool Character::CanMoveRL() const
-{
-  if (!IsImmobile() || IsFalling()) return false;
-  return rl_motion_pause < Time::GetInstance()->Read();
-}
-
-void Character::BeginMovementRL(uint pause, bool slowly)
-{
-  Camera::GetInstance()->FollowObject(this, true);
-
-  walking_time = Time::GetInstance()->Read();
-  UpdateLastMovingTime();
-  if (!slowly) {
-    SetMovement("walk");
-  }
-  CharacterCursor::GetInstance()->Hide();
-  step_sound_played = true;
-  rl_motion_pause = Time::GetInstance()->Read()+pause;
-}
-
-bool Character::CanStillMoveRL(uint pause)
-{
-  if (rl_motion_pause + pause < Time::GetInstance()->Read())
-  {
-    walking_time = Time::GetInstance()->Read();
-    rl_motion_pause = rl_motion_pause + pause;
-    return true;
-  }
-  return false;
-}
-
 // Signal the end of a fall
-void Character::SignalCollision(const Point2d& speed_vector)
+void Character::Collision(const Point2d& speed_vector)
 {
   // Do not manage dead characters.
   if (IsDead()) return;
@@ -679,15 +729,14 @@ void Character::SignalCollision(const Point2d& speed_vector)
   body->SetRotation(0.0);
   back_jumping = false;
 
-  double norm = speed_vector.Norm();
+  Double norm = speed_vector.Norm();
 
-  MSG_DEBUG("character.collision", "%s collides with speed %f, %f (norm = %f)",
-	    character_name.c_str(), speed_vector.x, speed_vector.y, norm);
-
-  if (norm > game_mode->safe_fall && speed_vector.y>0.0)
+  if (norm > game_mode->safe_fall && speed_vector.y > ZERO)
   {
+    // TODO: take the angle of collision into account!
+
     norm -= game_mode->safe_fall;
-    double degat = norm * game_mode->damage_per_fall_unit;
+    Double degat = norm * game_mode->damage_per_fall_unit;
     SetEnergyDelta (-(int)degat);
     Game::GetInstance()->SignalCharacterDamage(this);
     SetClothe("normal");
@@ -701,11 +750,37 @@ void Character::SignalCollision(const Point2d& speed_vector)
   }
 }
 
+void Character::SignalGroundCollision(const Point2d& speed_before)
+{
+  MSG_DEBUG("character.collision", "%s collides on ground with speed %s, %s (norm = %s)",
+            character_name.c_str(), 
+            Double2str(speed_before.x).c_str(), 
+            Double2str(speed_before.y).c_str(),
+            Double2str(speed_before.Norm()).c_str());
+
+  Collision(speed_before);
+}
+
+void Character::SignalObjectCollision(const Point2d& my_speed_before,
+				      PhysicalObj * /* obj */,
+				      const Point2d& /* obj_speed */)
+{
+  MSG_DEBUG("character.collision", "%s collides on object with speed %s, %s (norm = %s)",
+            character_name.c_str(),
+            Double2str(my_speed_before.x).c_str(), 
+            Double2str(my_speed_before.y).c_str(),
+            Double2str(my_speed_before.Norm()).c_str());
+  // In case an object collides with the character, we don't want
+  // the character to have huge damage because of the speed of the object.
+  // Damage should be applied when felt or when hurted by a weapon.
+  Collision(my_speed_before);
+}
+
 void Character::SignalExplosion()
 {
   if (IsDead()) return;
 
-  double n, a;
+  Double n, a;
   GetSpeed(n, a);
   SetRebounding(true);
 
@@ -726,7 +801,7 @@ void Character::SignalExplosion()
   Game::GetInstance()->SignalCharacterDamage(this);
 }
 
-BodyDirection_t Character::GetDirection() const
+LRDirection Character::GetDirection() const
 {
   return body->GetDirection();
 }
@@ -739,8 +814,11 @@ void Character::StopPlaying()
   if (IsDead()) return;
   SetClothe("normal");
   SetMovement("breathe");
-  body->ResetWalk();
+  if (IsWalking())
+    StopWalking();
   SetRebounding(true);
+  lr_move_intentions.clear();
+  ud_move_intentions.clear();
 }
 
 // Begining of turn or changed to this character
@@ -764,25 +842,34 @@ bool Character::IsActiveCharacter() const
 }
 
 // Hand position
-const Point2i & Character::GetHandPosition() const {
-  return body->GetHandPosition();
+void Character::GetHandPosition(Point2i & result) const
+{
+  GetRelativeHandPosition(result);
+  result += GetPosition();
 }
 
-double Character::GetFiringAngle() const {
+void Character::GetRelativeHandPosition(Point2i & result) const
+{
+  body->GetRelativeHandPosition(result);
+}
+
+Double Character::GetFiringAngle() const {
   if (GetDirection() == DIRECTION_LEFT)
     return InverseAngleRad(firing_angle);
   return firing_angle;
 }
 
-void Character::SetFiringAngle(double angle) {
-  /*while(angle > 2 * M_PI)
-    angle -= 2 * M_PI;
-  while(angle <= -2 * M_PI)
-    angle += 2 * M_PI;*/
+#include <iostream>
+void Character::SetFiringAngle(Double angle) {
+  /*while(angle > 2 * PI)
+    angle -= 2 * PI;
+  while(angle <= -2 * PI)
+    angle += 2 * PI;*/
   angle = InRange_Double(angle, -(ActiveTeam().GetWeapon().GetMaxAngle()),
                              -(ActiveTeam().GetWeapon().GetMinAngle()));
   firing_angle = angle;
   m_team.crosshair.Refresh(GetFiringAngle());
+  body->Rebuild();
 }
 
 void Character::SetWeaponClothe()
@@ -848,245 +935,268 @@ uint Character::GetCharacterIndex() const
   return 0;
 }
 
-// ###################################################################
-// ###################################################################
-// ###################################################################
-
-void Character::StoreValue(Action *a)
-{
-  PhysicalObj::StoreValue(a);
-  a->Push((int)GetDirection());
-  a->Push(GetAbsFiringAngle());
-  a->Push((int)disease_damage_per_turn);
-  a->Push((int)disease_duration);
-  if (IsActiveCharacter()) { // If active character, store step animation
-    a->Push((int)true);
-    a->Push(GetBody()->GetClothe());
-    a->Push(GetBody()->GetMovement());
-    a->Push((int)GetBody()->GetFrame());
-  } else {
-    a->Push((int)false);
-  }
-}
-
-void Character::GetValueFromAction(Action *a)
-{
-  // those 2 parameters will be retrieved by PhysicalObj::GetValueFromAction
-  alive_t prev_live_state = m_alive;
-  int prev_energy = m_energy;
-  Point2d prev_position = Physics::GetPos();
-
-  PhysicalObj::GetValueFromAction(a);
-  SetDirection((BodyDirection_t)(a->PopInt()));
-  SetFiringAngle(a->PopDouble());
-
-  if (m_alive != prev_live_state) {
-    switch (m_alive) {
-    case ALIVE:
-      fprintf(stderr, "Character::GetValueFromAction: %s has been resurrected\n",
-	      GetName().c_str());
-      SetClothe("normal");
-      SetMovement("breathe");
-      if (prev_live_state == DROWNED) {
-	SignalGoingOutOfWater();
-      }
-      break;
-    case DEAD:
-      fprintf(stderr,
-	      "Character::GetValueFromAction: %s has died on the other side of the network\n"
-	      "        Previous energy: %d\n",
-	      GetName().c_str(), prev_energy);
-      death_explosion = false;
-
-      // to avoid violating an ASSERT in Die()
-      m_alive = prev_live_state;
-      if (m_alive != ALIVE && m_alive != DROWNED)
-	m_alive = ALIVE;
-
-      Die();
-      break;
-    case GHOST: {
-      fprintf(stderr, "Character::GetValueFromAction: %s is now a ghost!\n", GetName().c_str());
-      m_alive = prev_live_state;
-      bool was_dead = IsDead();
-      m_alive = GHOST;
-      SignalGhostState(was_dead);
-      break;
-    }
-    case DROWNED:
-      fprintf(stderr, "Character::GetValueFromAction: %s is drowning!\n", GetName().c_str());
-      SignalDrowning();
-      break;
-    }
-  }
-
-  if (prev_energy != m_energy) {
-    fprintf(stderr,
-	    "Character::GetValueFromAction: energy points were differents for %s:\n"
-	    "        - remote : %d\n"
-	    "        - local  : %d\n",
-	    GetName().c_str(), m_energy, prev_energy);
-    if (m_energy > 0) {
-      energy_bar.Actu(m_energy);
-    }
-  }
-
-  uint disease_damage_per_turn = (a->PopInt());
-  uint disease_duration = (a->PopInt());
-  SetDiseaseDamage(disease_damage_per_turn, disease_duration);
-  if (a->PopInt()) { // If active characters, retrieve stored animation
-    if (GetTeam().IsActiveTeam())
-      ActiveTeam().SelectCharacter(this);
-    SetClothe(a->PopString());
-    SetMovement(a->PopString());
-    GetBody()->SetFrame((uint)(a->PopInt()));
-
-    GetBody()->UpdateWeaponPosition(GetPosition());
-  }
-
-  // If the player has moved, the camera should follow it!
-  Point2d current_position = Physics::GetPos();
-  if (IsActiveCharacter() && prev_position != current_position) {
-    Camera::GetInstance()->FollowObject(this, true);
-    HideGameInterface();
-  }
-}
-
-
 const std::string& Character::GetName() const
 {
     return character_name;
- }
+}
 
 void Character::SetCustomName(const std::string name)
 {
-  std::cout<<"Character::SetCustomName "<<name<<std::endl;
-
-  if(name.size()>0)
+  if (!name.empty())
   {
-    name_text->Set(name);
+    name_text->SetText(name);
     character_name = name;
   }
-
-
 }
+
 // ###################################################################
 // ###################################################################
 // ###################################################################
 
-// #################### MOVE_RIGHT
-void Character::HandleKeyPressed_MoveRight(bool shift)
+void Character::StartOrStopWalkingIfNecessary()
 {
-  BeginMovementRL(GameMode::GetInstance()->character.walking_pause, shift);
-  body->StartWalk();
-
-  HandleKeyRefreshed_MoveRight(shift);
+  const LRMoveIntention * lr_move_intention = GetLastLRMoveIntention();
+  bool should_walk = (lr_move_intention != NULL)
+    && !GetTeam().AccessWeapon().IsPreventingLRMovement()
+    && HasGroundUnderFeets()
+    && !IsDead();
+  if (should_walk) {
+    if (lr_move_intention->GetDirection() != GetDirection() && !IsChangingDirection()) {
+      SetDirection(lr_move_intention->GetDirection());
+      last_direction_change = Time::GetInstance()->Read();
+    }
+  }
+  if (should_walk && !IsChangingDirection()) {
+    bool should_be_slowly = lr_move_intention->IsToDoItSlowly();
+    if (IsWalking() && (should_be_slowly != walking_slowly))
+      StopWalking();
+    if (!IsWalking())
+      StartWalking(should_be_slowly);
+  } else {
+    if (IsWalking())
+      StopWalking();
+  }
 }
 
-void Character::HandleKeyRefreshed_MoveRight(bool shift) const
+void Character::StartWalking(bool slowly)
 {
-  HideGameInterface();
+  walking_time = Time::GetInstance()->Read();
+  rl_motion_pause = max(rl_motion_pause , Time::GetInstance()->Read());
+  step_sound_played = true;
+  walking_slowly = slowly;
 
-  if (ActiveCharacter().IsImmobile())
-    MoveActiveCharacterRight(shift);
+  Camera::GetInstance()->FollowObject(this);
+  if (Network::GetInstance()->IsTurnMaster()) {
+    HideGameInterface();
+    ActiveTeam().crosshair.Hide();
+  }
+  CharacterCursor::GetInstance()->Hide();
+  UpdateLastMovingTime();
+  Game::GetInstance()->SetCharacterChosen(true);
+
+  if (!slowly)
+    SetMovement("walk");
+  body->StartWalking();
 }
 
-void Character::HandleKeyReleased_MoveRight(bool)
+void Character::StopWalking()
 {
-  body->StopWalk();
-  SendActiveCharacterInfo();
+  if (Network::GetInstance()->IsTurnMaster())
+    ActiveTeam().crosshair.Show();
+  body->StopWalking();
 }
 
-// #################### MOVE_LEFT
-void Character::HandleKeyPressed_MoveLeft(bool shift)
+bool Character::IsWalking() const
 {
-  BeginMovementRL(GameMode::GetInstance()->character.walking_pause, shift);
-  body->StartWalk();
-
-  HandleKeyRefreshed_MoveLeft(shift);
+  return body->IsWalking();
 }
 
-void Character::HandleKeyRefreshed_MoveLeft(bool shift) const
+void Character::MakeSteps()
 {
-  HideGameInterface();
+  int height;
+  bool ghost;
+  uint walking_pause = GameMode::GetInstance()->character.walking_pause;
 
-  if (ActiveCharacter().IsImmobile())
-    MoveActiveCharacterLeft(shift);
+  const LRMoveIntention * lr_move_intention = GetLastLRMoveIntention();
+  ASSERT(lr_move_intention != NULL);
+  if (lr_move_intention->IsToDoItSlowly())
+    walking_pause *= 10;
+  else
+    SetMovement("walk");// otherwise character would slide after dropping a dynamite
+
+  // If character moves out of the world, no need to go further: it is dead
+  if (GetDirection() == DIRECTION_LEFT)
+    ghost = IsOutsideWorld ( Point2i(-1, 0) );
+  else
+    ghost = IsOutsideWorld ( Point2i(1, 0) );
+
+  if (ghost) {
+    MSG_DEBUG("ghost", "%s will be a ghost.", GetName().c_str());
+    Ghost();
+    return;
+  }
+
+  // Check we can move (to go not too fast)
+  while ((rl_motion_pause + walking_pause < Time::GetInstance()->Read()) &&
+         ComputeHeightMovement(height)) {
+    walking_time = Time::GetInstance()->Read();
+    rl_motion_pause = rl_motion_pause + walking_pause;
+
+    // Eventually moves the character
+    SetXY( Point2i(GetX() + GetDirection(), GetY() + height));
+
+    // If no collision, let gravity do its job
+    UpdatePosition();
+  }
 }
 
-void Character::HandleKeyReleased_MoveLeft(bool)
+bool Character::IsChangingDirection()
 {
-  body->StopWalk();
-  SendActiveCharacterInfo();
+  return last_direction_change + PAUSE_CHG_DIRECTION >= Time::GetInstance()->Read();
+}
+
+bool Character::ComputeHeightMovement(int & height)
+{
+  if (IsInVacuum(Point2i(GetDirection(), 0))
+      && !IsInVacuum(Point2i(GetDirection(), +1)) ){
+    //Land is flat, we can move!
+    height = 0;
+    return true;
+  }
+
+  //Compute height of the step:
+  if (IsInVacuum(Point2i(GetDirection(), 0))) {
+    //Try to go down:
+    for (height = 2; height <= MAX_FALLING_HEIGHT ; height++) {
+      if (!IsInVacuum(Point2i(GetDirection(), height))) {
+        height--;
+        return true;
+      }
+    }
+
+    //We can go down, but the step is too big -> the character will fall
+    bool falling = true;
+    if (falling) {
+      SetX (GetXDouble() + GetDirection());
+      UpdatePosition();
+      SetMovement("fall");
+    }
+    return false;
+  } else {
+    //Try to go up:
+    for (height = -1; height >= -MAX_CLIMBING_HEIGHT ; height--) {
+      if (IsInVacuum(Point2i(GetDirection(), height))) {
+        return true;
+      }
+    }
+  }
+  //We can't move!
+  return false;
+}
+
+void Character::HandleKeyPressed_MoveRight(bool slowly)
+{
+  Action *a = new Action(Action::ACTION_CHARACTER_START_MOVING_RIGHT);
+  a->Push(slowly ? 1 : 0);
+  ActionHandler::GetInstance()->NewAction(a);
+}
+
+void Character::HandleKeyReleased_MoveRight(bool slowly)
+{
+  Action *a = new Action(Action::ACTION_CHARACTER_STOP_MOVING_RIGHT);
+  a->Push(slowly ? 1 : 0);
+  ActionHandler::GetInstance()->NewAction(a);
+}
+
+void Character::HandleKeyPressed_MoveLeft(bool slowly)
+{
+  Action *a = new Action(Action::ACTION_CHARACTER_START_MOVING_LEFT);
+  a->Push(slowly ? 1 : 0);
+  ActionHandler::GetInstance()->NewAction(a);
+}
+
+void Character::HandleKeyReleased_MoveLeft(bool slowly)
+{
+  Action *a = new Action(Action::ACTION_CHARACTER_STOP_MOVING_LEFT);
+  a->Push(slowly ? 1 : 0);
+  ActionHandler::GetInstance()->NewAction(a);
 }
 
 // #################### UP
-void Character::HandleKeyRefreshed_Up(bool shift)
+
+void Character::HandleKeyPressed_Up(bool slowly)
 {
-  HideGameInterface();
-  if (ActiveCharacter().IsImmobile())
-    {
-      if (ActiveTeam().crosshair.enable)
-        {
-	  UpdateLastMovingTime();
-          CharacterCursor::GetInstance()->Hide();
-          if (shift) AddFiringAngle(-DELTA_CROSSHAIR/10.0);
-          else       AddFiringAngle(-DELTA_CROSSHAIR);
-          SendActiveCharacterInfo();
-        }
-    }
+  Action *a = new Action(Action::ACTION_CHARACTER_START_MOVING_UP);
+  a->Push(slowly ? 1 : 0);
+  ActionHandler::GetInstance()->NewAction(a);
+}
+
+void Character::HandleKeyReleased_Up(bool slowly)
+{
+  Action *a = new Action(Action::ACTION_CHARACTER_STOP_MOVING_UP);
+  a->Push(slowly ? 1 : 0);
+  ActionHandler::GetInstance()->NewAction(a);
 }
 
 // #################### DOWN
-void Character::HandleKeyRefreshed_Down(bool shift)
+
+void Character::HandleKeyPressed_Down(bool slowly)
 {
-  HideGameInterface();
-  if(ActiveCharacter().IsImmobile())
-    {
-      if (ActiveTeam().crosshair.enable)
-        {
-	  UpdateLastMovingTime();
-          CharacterCursor::GetInstance()->Hide();
-          if (shift) AddFiringAngle(DELTA_CROSSHAIR/10.0);
-          else       AddFiringAngle(DELTA_CROSSHAIR);
-          SendActiveCharacterInfo();
-        }
-    }
+  Action *a = new Action(Action::ACTION_CHARACTER_START_MOVING_DOWN);
+  a->Push(slowly ? 1 : 0);
+  ActionHandler::GetInstance()->NewAction(a);
+}
+
+void Character::HandleKeyReleased_Down(bool slowly)
+{
+  Action *a = new Action(Action::ACTION_CHARACTER_STOP_MOVING_DOWN);
+  a->Push(slowly ? 1 : 0);
+  ActionHandler::GetInstance()->NewAction(a);
 }
 
 // #################### JUMP
 
-void Character::HandleKeyPressed_Jump(bool)
+void Character::HandleKeyPressed_Jump()
 {
+  if (ActiveTeam().AccessWeapon().IsPreventingJumps())
+    return;
   HideGameInterface();
-  if (ActiveCharacter().IsImmobile()) {
-    Jump();
-    Action a(Action::ACTION_CHARACTER_JUMP);
-    SendActiveCharacterAction(a);
+
+  ActiveTeam().crosshair.Hide();
+
+  if (IsImmobile()) {
+    Action *a = new Action(Action::ACTION_CHARACTER_JUMP);
+    ActionHandler::GetInstance()->NewAction(a);
   }
 }
 
 // #################### HIGH JUMP
-void Character::HandleKeyPressed_HighJump(bool)
+void Character::HandleKeyPressed_HighJump()
 {
+  if (ActiveTeam().AccessWeapon().IsPreventingJumps())
+    return;
   HideGameInterface();
-  if (ActiveCharacter().IsImmobile()) {
-    HighJump();
-    Action a(Action::ACTION_CHARACTER_HIGH_JUMP);
-    SendActiveCharacterAction(a);
+
+  ActiveTeam().crosshair.Hide();
+
+  if (IsImmobile()) {
+    Action *a = new Action(Action::ACTION_CHARACTER_HIGH_JUMP);
+    ActionHandler::GetInstance()->NewAction(a);
   }
 }
 
 // #################### BACK JUMP
-void Character::HandleKeyPressed_BackJump(bool)
+void Character::HandleKeyPressed_BackJump()
 {
+  if (ActiveTeam().AccessWeapon().IsPreventingJumps())
+    return;
   HideGameInterface();
-  if (ActiveCharacter().IsImmobile()) {
-    BackJump();
-    Action a(Action::ACTION_CHARACTER_BACK_JUMP);
-    SendActiveCharacterAction(a);
+
+  ActiveTeam().crosshair.Hide();
+
+  if (IsImmobile()) {
+    Action *a = new Action(Action::ACTION_CHARACTER_BACK_JUMP);
+    ActionHandler::GetInstance()->NewAction(a);
   }
 }
 
-
-
