@@ -1,6 +1,6 @@
 /******************************************************************************
  *  Wormux is a convivial mass murder game.
- *  Copyright (C) 2001-2008 Wormux Team.
+ *  Copyright (C) 2001-2010 Wormux Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,14 +23,11 @@
 #define GAME_CLASSIC_H
 
 #include "game/game.h"
-#include "sound/sound_sample.h"
 
 class GameClassic : public Game
 {
   uint duration;
 
-  SoundSample countdown_sample;
-
   /* If you need this, implement it (correctly)*/
   GameClassic(const GameClassic&);
   GameClassic operator=(const GameClassic&);
