@@ -1,6 +1,6 @@
 /******************************************************************************
  *  Wormux is a convivial mass murder game.
- *  Copyright (C) 2001-2009 Wormux Team.
+ *  Copyright (C) 2001-2010 Wormux Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
 //wave_per is the number of periods of the wave when it is waved at the maximum
 //used on the skin during teleportation
 Sprite* WaveSurface(Surface &a, unsigned int nbr_frames,
-                    unsigned int duration, float wave_amp, float wave_per){
+                    unsigned int duration, Double wave_amp, Double wave_per){
   Sprite* sprite = new Sprite;
   Point2i newSize = a.GetSize() + Point2i(2 * (int)wave_amp, 0);
 
@@ -46,8 +46,8 @@ Sprite* WaveSurface(Surface &a, unsigned int nbr_frames,
         a.GetRGBA(col, r, g, bl, al);
         col = b.MapRGBA(r, g, bl, al);
 
-        float t = (float)nbr_frames * sin(M_PI*(float)f/(float)nbr_frames);
-        unsigned int wave_x = (unsigned int)(x+(wave_amp*(1+sin(((float)t*wave_per*2.0*M_PI/(float)nbr_frames)*(float)y*2.0*M_PI/(float)a.GetHeight()))));
+        Double t = (Double)nbr_frames * sin(PI*(Double)f/(Double)nbr_frames);
+        unsigned int wave_x = (long)(x+(wave_amp*(1+sin(((Double)t*wave_per*TWO*PI/(Double)nbr_frames)*(Double)y*TWO*PI/(Double)a.GetHeight()))));
         b.PutPixel(wave_x, y, col);
       }
     }
@@ -66,26 +66,26 @@ Sprite* WaveSurface(Surface &a, unsigned int nbr_frames,
 // dy_max offset max of the rebound ( 0 <= dy <= dy_max )
 void Rebound(Sprite* spr, int &dy, uint t0, uint per, int dy_max)
 {
-  float scale_x, scale_y;
+  Double scale_x, scale_y;
   uint dt = (Time::GetInstance()->Read() - t0) % per;
 
-  spr->Scale(1.0,1.0);
+  spr->Scale(ONE,ONE);
   dy = 0;
 
   //sprite at bottom:
   if( dt < per / 4 )
   {
-    float dt2 = ((per / 4) - dt) / ((float)per / 4.0);
-    scale_y =        2.0*dt2*dt2 - 2.0*dt2 + 1.0;
-    scale_x = 2.0 - (2.0*dt2*dt2 - 2.0*dt2 + 1.0);
+    Double dt2 = ((Double)((per / 4) - dt)) / ((Double)per / FOUR);
+    scale_y =        TWO*dt2*dt2 - TWO*dt2 + ONE;
+    scale_x = TWO - (TWO*dt2*dt2 - TWO*dt2 + ONE);
     dy = 0;
     spr->Scale(scale_x,scale_y);
     return;
   }
 
   dt -= per/4;
-  float dt2 = ((3*per/4)-dt)/(3.0*per/4.0);
-  dy += (int)((-4.0*dt2*dt2 + 4.0*dt2) * dy_max);
+  Double dt2 = ((Double)((3*per/4)-dt))/(THREE*per/FOUR);
+  dy += (int)((-FOUR*dt2*dt2 + FOUR*dt2) * dy_max);
 }
 
 //Returns value of y_stretch and y, to be used on an object, to make him do
@@ -109,7 +109,7 @@ void Gelatine (int &y, int &stretch_y, uint t0, uint amp, uint dur, uint per)
   amp = amp * (dur - dt) / dur;
 
   //Scale
-  stretch_y = (int)(sin((float)per * (float)dt * 2.0 * M_PI / (float)dur) * (float)amp);
+  stretch_y = (int)(sin((Double)per * (Double)dt * TWO * PI / (Double)dur) * (Double)amp);
 
   //Offset
   if(stretch_y < 0.0)
