@@ -1,6 +1,6 @@
 /******************************************************************************
  *  Wormux is a convivial mass murder game.
- *  Copyright (C) 2001-2008 Wormux Team.
+ *  Copyright (C) 2001-2010 Wormux Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,14 +19,14 @@
  * Handle a SDL Surface
  *****************************************************************************/
 
-#include "graphic/surface.h"
 #include <SDL.h>
 #include <SDL_gfxPrimitives.h>
 #include <SDL_image.h>
 #include <SDL_rotozoom.h>
 #include <iostream>
 #include <png.h>
-#include "tool/i18n.h"
+
+#include "graphic/surface.h"
 #include "tool/math_tools.h"
 
 /* texturedPolygon import from SDL_gfx v2.0.15 */
@@ -305,11 +305,11 @@ int Surface::Blit(const Surface& src, const Rectanglei &srcRect, const Point2i &
  * @param spr
  * @param position
  */
-void Surface::MergeSurface(Surface &spr, const Point2i &pos)
+void Surface::MergeSurface( Surface &spr, const Point2i &pos)
 {
   Uint32 spr_pix, cur_pix;
   Uint8 r, g, b, a, p_r, p_g, p_b, p_a;
-  double f_a, f_ca, f_pa;
+  Double f_a, f_ca, f_pa;
   SDL_PixelFormat * current_fmt = surface->format;
   SDL_PixelFormat * spr_fmt = spr.surface->format;
   int current_offset, spr_offset;
@@ -317,34 +317,43 @@ void Surface::MergeSurface(Surface &spr, const Point2i &pos)
 
   spr.Lock();
   Lock();
+
   // for each pixel lines of a source image
   for (offset.x = (pos.x > 0 ? 0 : -pos.x); offset.x < spr.GetWidth() && pos.x + offset.x < GetWidth(); offset.x++) {
     for (offset.y = (pos.y > 0 ? 0 : -pos.y); offset.y < spr.GetHeight() && pos.y + offset.y < GetHeight(); offset.y++) {
+
       // Computing offset on both sprite
       current_offset = (pos.y + offset.y) * surface->w + pos.x + offset.x;
       spr_offset = offset.y * spr.surface->w + offset.x;
+
       // Retrieving a pixel of sprite to merge
       spr_pix = ((Uint32*)spr.surface->pixels)[spr_offset];
       cur_pix = ((Uint32*)surface->pixels)[current_offset];
+
       // Retreiving each chanel of the pixel using pixel format
       r = (Uint8)(((spr_pix & spr_fmt->Rmask) >> spr_fmt->Rshift) << spr_fmt->Rloss);
       g = (Uint8)(((spr_pix & spr_fmt->Gmask) >> spr_fmt->Gshift) << spr_fmt->Gloss);
       b = (Uint8)(((spr_pix & spr_fmt->Bmask) >> spr_fmt->Bshift) << spr_fmt->Bloss);
       a = (Uint8)(((spr_pix & spr_fmt->Amask) >> spr_fmt->Ashift) << spr_fmt->Aloss);
+
       // Retreiving previous alpha value
       p_a = (Uint8)(((cur_pix & current_fmt->Amask) >> current_fmt->Ashift) << current_fmt->Aloss);
-      if(a == SDL_ALPHA_OPAQUE || (p_a == 0 && a >0)) // new pixel with no alpha or nothing on previous pixel
+
+      if (a == SDL_ALPHA_OPAQUE || (p_a == 0 && a >0)) // new pixel with no alpha or nothing on previous pixel
         ((Uint32 *)(surface->pixels))[current_offset] = SDL_MapRGBA(current_fmt, r, g, b, a);
       else if (a > 0) { // alpha is lower => merge color with previous value
-        f_a = (double)a / 255.0;
-        f_ca = 1.0 - f_a;
-        f_pa = (double)p_a / 255.0;
+        f_a = (Double)a / (Double)255.0;
+        f_ca = ONE - f_a;
+        f_pa = (Double)p_a / (Double)255.0;
+
         p_r = (Uint8)(((cur_pix & current_fmt->Rmask) >> current_fmt->Rshift) << current_fmt->Rloss);
         p_g = (Uint8)(((cur_pix & current_fmt->Gmask) >> current_fmt->Gshift) << current_fmt->Gloss);
         p_b = (Uint8)(((cur_pix & current_fmt->Bmask) >> current_fmt->Bshift) << current_fmt->Bloss);
-        r = (Uint8)((double)p_r * f_ca * f_pa + (double)r * f_a);
-        g = (Uint8)((double)p_g * f_ca * f_pa + (double)g * f_a);
-        b = (Uint8)((double)p_b * f_ca * f_pa + (double)b * f_a);
+
+        r = (int)((Double)p_r * f_ca * f_pa + (Double)r * f_a);
+        g = (int)((Double)p_g * f_ca * f_pa + (Double)g * f_a);
+        b = (int)((Double)p_b * f_ca * f_pa + (Double)b * f_a);
+
         a = (a > p_a ? a : p_a);
         ((Uint32 *)(surface->pixels))[current_offset] = SDL_MapRGBA(current_fmt, r, g, b, a);
       }
@@ -435,7 +444,7 @@ void Surface::Flip()
   SDL_Flip( surface );
 }
 
-int Surface::BoxColor(const Rectanglei &rect, const Color &color)
+  int Surface::BoxColor(const Rectanglei &rect, const Color &color)
 {
   if( rect.IsSizeZero() )
     return 0;
@@ -711,8 +720,8 @@ end:
 * Warning rotozoomSurfaceXY uses degrees so the rotation of image use degrees here,
 * but when accessing thanks to GetSurfaceForAngle the index is using radian
 * (because we juste need an index in array, not an angle) */
-static const double ratio_deg_to_rad = 180 / M_PI;
-Surface Surface::RotoZoom(double angle, double zoomx, double zoomy, int smooth)
+static const Double ratio_deg_to_rad = 180 / PI;
+Surface Surface::RotoZoom(Double angle, Double zoomx, Double zoomy, int smooth)
 {
   SDL_Surface *surf;
 
@@ -722,16 +731,16 @@ Surface Surface::RotoZoom(double angle, double zoomx, double zoomy, int smooth)
    * can also be negative. In this case the corresponding axis is flipped.
    * Note: Flipping currently only works with antialiasing turned off
    */
-  if (zoomx < 0.0 || zoomy < 0.0)
+  if (zoomx < ZERO || zoomy < ZERO)
     smooth = SMOOTHING_OFF;
 #endif
 
   if (EqualsZero(angle))
-    surf = zoomSurface(surface, zoomx, zoomy, smooth);
-  else if (zoomx == zoomy && zoomx > 0.0)
-    surf = rotozoomSurface(surface, angle * ratio_deg_to_rad , zoomx, smooth);
+    surf = zoomSurface(surface, zoomx.toDouble(), zoomy.toDouble(), smooth);
+  else if (zoomx == zoomy && zoomx > ZERO)
+    surf = rotozoomSurface(surface, (angle * ratio_deg_to_rad).toDouble() , zoomx.toDouble(), smooth);
   else
-    surf = rotozoomSurfaceXY(surface, angle * ratio_deg_to_rad , zoomx, zoomy, smooth);
+    surf = rotozoomSurfaceXY(surface, (angle * ratio_deg_to_rad).toDouble() , zoomx.toDouble(), zoomy.toDouble(), smooth);
 
   if(!surf)
     Error( "Unable to make a rotozoom on the surface !" );
@@ -901,3 +910,59 @@ Uint32 Surface::ComputeCRC()
   Unlock();
   return crc;
 }
+
+
+/**
+ * Set an alpha . This's alpha is set to red value off mask
+ *
+ * @param mask
+ * @param position
+ */
+void Surface::MergeAlphaSurface(const Surface &mask, const Point2i &pos)
+{
+  int dest_x = pos.x;
+  int dest_y = pos.y;
+  int size_x = mask.GetWidth();
+  int size_y = mask.GetHeight();
+  int mask_x = 0;
+  int mask_y = 0;
+
+  if(dest_x<0){
+    dest_x = 0;
+    mask_x = -pos.x;
+	}
+  if(dest_y<0){
+    dest_y = 0;
+    mask_y = -pos.y;
+  }
+
+  if((size_x + dest_x) > GetWidth()){
+    size_x = GetWidth()- dest_x;
+  }
+
+  if((size_y + dest_y) > GetHeight()){
+    size_y = GetHeight() - dest_y;
+  }
+  Uint32 temp_pix;
+
+  Uint32 mask_color, this_color;
+  Uint8 mask_r, mask_g, mask_b, mask_a;
+  Uint8 this_r, this_g, this_b, this_a;
+
+  for(int i = 0; i<size_x;i++){
+    for(int j = 0;j<size_y;j++){
+      //Copy red pixel of mask in alpha pixel of this
+      mask_color  = mask.GetPixel(i + mask_x, j + mask_y);
+      this_color = GetPixel(i + dest_x, j + dest_y) ;
+
+      GetRGBA(this_color, this_r, this_g, this_b, this_a);
+      mask.GetRGBA(mask_color,mask_r,mask_g,mask_b,mask_a);
+
+      if(mask_r < this_a){
+        temp_pix = MapRGBA(this_r, this_g, this_b, mask_r);
+        PutPixel(i+ dest_x, j + dest_y, temp_pix);
+      }
+
+    }
+  }
+}
