@@ -1,6 +1,6 @@
 /******************************************************************************
  *  Wormux is a convivial mass murder game.
- *  Copyright (C) 2001-2008 Wormux Team.
+ *  Copyright (C) 2001-2010 Wormux Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -34,9 +34,10 @@ class LoadingScreen
  private:
   Profile * res;
   Sprite * loading_bg;
+  int icon_count;
 
  public:
-  LoadingScreen();
+  LoadingScreen(int icon_count);
   ~LoadingScreen();
 
   void DrawBackground();
