@@ -1,6 +1,6 @@
 /******************************************************************************
  *  Wormux is a convivial mass murder game.
- *  Copyright (C) 2001-2008 Wormux Team.
+ *  Copyright (C) 2001-2010 Wormux Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,14 +29,13 @@
 #include "gui/text_box.h"
 #include "team/teams_list.h"
 #include "team/team.h"
-#include "tool/i18n.h"
 
 #include <iostream>
 
-TeamsSelectionBox::TeamsSelectionBox(const Point2i &_size, bool network) :
-  HBox(_size.y, network)
+TeamsSelectionBox::TeamsSelectionBox(const Point2i &_size, bool network, bool w_border) :
+  HBox(_size.y, w_border)
 {
-  if (!network)
+  if (!w_border)
     SetNoBorder();
 
   // How many teams ?
@@ -105,6 +104,7 @@ TeamsSelectionBox::TeamsSelectionBox(const Point2i &_size, bool network) :
     if (j < 2) {
       SetNbTeams(2);
       local_teams_nb->SetValue(2);
+      teams_selections.at(1)->SetAIName(DEFAULT_AI_NAME);
     } else {
       local_teams_nb->SetValue(j);
     }
@@ -128,15 +128,23 @@ Widget* TeamsSelectionBox::ClickUp(const Point2i &mousePosition, uint button)
         if (w == NULL) {
           Rectanglei r(teams_selections.at(i)->GetPositionX(),
                        teams_selections.at(i)->GetPositionY(),
-                       60,
-                       60);
+                       38,
+                       38);
           if ( r.Contains(mousePosition) ) {
-            if ( button == SDL_BUTTON_LEFT || button == SDL_BUTTON_WHEELDOWN ) {
+            if ( button == Mouse::BUTTON_LEFT() || button == SDL_BUTTON_WHEELDOWN ) {
               NextTeam(i);
-            } else if ( button == SDL_BUTTON_RIGHT || button == SDL_BUTTON_WHEELUP ) {
+            } else if ( button == Mouse::BUTTON_RIGHT() || button == SDL_BUTTON_WHEELUP ) {
               PrevTeam(i);
             }
-          }
+          } else {
+	    Rectanglei r2(teams_selections.at(i)->GetPositionX(),
+			  teams_selections.at(i)->GetPositionY() + 39,
+			  38,
+			  30);
+	    if (r2.Contains(mousePosition)) {
+	      teams_selections.at(i)->SwitchPlayerType();
+	    }
+	  }
         } else {
           return w;
         }
