@@ -1,6 +1,6 @@
 /******************************************************************************
  *  Wormux is a convivial mass murder game.
- *  Copyright (C) 2001-2008 Wormux Team.
+ *  Copyright (C) 2001-2010 Wormux Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,19 +22,21 @@
 #ifndef RANDOMSYNC_H
 #define RANDOMSYNC_H
 
-#include "include/singleton.h"
-#include "tool/random.h"
+#include <WORMUX_singleton.h>
+#include <WORMUX_random.h>
 
 class RandomSyncGen : public RandomGenerator, public Singleton<RandomSyncGen>
 {
+private:
 #ifdef DEBUG
   uint nb_get;
 #endif
-
+protected:
+  virtual uint GetRand();
+  virtual void SetRand(uint seed);
 public:
-  void InitRandom();
-  uint GetRand();
-  void SetRand(uint seed);
+  virtual void InitRandom();
+  void Verify();
 };
 
 RandomSyncGen& RandomSync();
