@@ -1,6 +1,6 @@
 /******************************************************************************
  *  Wormux is a convivial mass murder game.
- *  Copyright (C) 2001-2008 Wormux Team.
+ *  Copyright (C) 2001-2010 Wormux Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -38,29 +38,35 @@
 #include "team/macro.h"
 #include "team/team.h"
 #include "team/teams_list.h"
-#include "tool/debug.h"
+#include <WORMUX_debug.h>
 #include "tool/isnan.h"
 #include "tool/math_tools.h"
-#include "tool/point.h"
-#include "tool/random.h"
-#include "tool/rectangle.h"
+#include "tool/string_tools.h"
+#include <WORMUX_point.h>
+#include <WORMUX_random.h>
+#include <WORMUX_rectangle.h>
 #include "weapon/weapon_launcher.h"
 
 const int Y_OBJET_MIN = -10000;
-const int WATER_RESIST_FACTOR = 40;
+const Double PIXEL_PER_METER = 40;
 
-const double PIXEL_PER_METER = 40;
-
-double MeterDistance (const Point2i &p1, const Point2i &p2)
+Double MeterDistance (const Point2i &p1, const Point2i &p2)
 {
   return p1.Distance(p2) / PIXEL_PER_METER;
 }
 
 PhysicalObj::PhysicalObj (const std::string &name, const std::string &xml_config) :
-  m_goes_through_wall(false),
+  m_collides_with_ground(true),
   m_collides_with_characters(false),
   m_collides_with_objects(false),
   m_rebound_position(-1,-1),
+  m_test_left(0),
+  m_test_right(0),
+  m_test_top(0),
+  m_test_bottom(0),
+  m_width(0),
+  m_height(0),
+  can_be_ghost(true),
   // No collision with this object until we have gone out of his collision rectangle
   m_overlapping_object(NULL),
   m_minimum_overlapse_time(0),
@@ -68,12 +74,6 @@ PhysicalObj::PhysicalObj (const std::string &name, const std::string &xml_config
   m_is_character(false),
   m_is_fire(false),
   m_name(name),
-  m_test_left(0),
-  m_test_right(0),
-  m_test_top(0),
-  m_test_bottom(0),
-  m_width(0),
-  m_height(0),
   m_rebound_sound(""),
   m_alive(ALIVE),
   m_energy(-1),
@@ -96,14 +96,14 @@ PhysicalObj::~PhysicalObj ()
 
 void PhysicalObj::SetXY(const Point2i &position)
 {
-  SetXY(Point2d(double(position.x), double(position.y)));
+  SetXY(Point2d(Double(position.x), Double(position.y)));
 }
 
 void PhysicalObj::SetXY(const Point2d &position)
 {
   CheckOverlapping();
 
-  if( IsOutsideWorldXY( Point2i(int(position.x), int(position.y)) ) )
+  if( IsOutsideWorldXY( Point2i(int(position.x), int(position.y)) ) && can_be_ghost )
     {
       SetPhysXY( position / PIXEL_PER_METER );
       Ghost();
@@ -116,60 +116,22 @@ void PhysicalObj::SetXY(const Point2d &position)
     }
 }
 
-double PhysicalObj::GetXdouble() const { return round(GetPhysX() * PIXEL_PER_METER); };
-double PhysicalObj::GetYdouble() const { return round(GetPhysY() * PIXEL_PER_METER); };
+Double PhysicalObj::GetXDouble() const { return round(GetPhysX() * PIXEL_PER_METER); };
+Double PhysicalObj::GetYDouble() const { return round(GetPhysY() * PIXEL_PER_METER); };
 
-int PhysicalObj::GetX() const { return (int)GetXdouble(); };
-int PhysicalObj::GetY() const { return (int)GetYdouble(); };
+int PhysicalObj::GetX() const { return (int)GetXDouble(); };
+int PhysicalObj::GetY() const { return (int)GetYDouble(); };
 
-void PhysicalObj::SetSize(const Point2i &newSize){
+void PhysicalObj::SetSize(const Point2i &newSize)
+{
   if( newSize == Point2i(0, 0) )
           Error( "New size of (0, 0) !");
   m_width = newSize.x;
   m_height = newSize.y;
-  SetPhysSize( (double)newSize.x / PIXEL_PER_METER, (double)newSize.y/PIXEL_PER_METER );
-}
 
-void PhysicalObj::StoreValue(Action *a)
-{
-  Physics::StoreValue(a);
-  a->Push(m_goes_through_wall);
-  a->Push(m_collides_with_characters);
-  a->Push(m_collides_with_objects);
-  a->Push(m_rebound_position);
-  a->Push((int)m_minimum_overlapse_time);
-  a->Push(m_ignore_movements);
-  a->Push(m_is_character);
-  a->Push((int)m_test_left);
-  a->Push((int)m_test_right);
-  a->Push((int)m_test_top);
-  a->Push((int)m_test_bottom);
-  a->Push((int)m_width);
-  a->Push((int)m_height);
-  a->Push((int)m_alive);
-  a->Push(m_energy);
-  a->Push(m_allow_negative_y);
-}
-
-void PhysicalObj::GetValueFromAction(Action *a)
-{
-  Physics::GetValueFromAction(a);
-  m_goes_through_wall        = !!a->PopInt();
-  m_collides_with_characters = !!a->PopInt();
-  m_collides_with_objects    = !!a->PopInt();
-  m_rebound_position         = a->PopPoint2i();
-  m_minimum_overlapse_time   = (uint)a->PopInt();
-  m_ignore_movements         = !!a->PopInt();
-  m_is_character             = !!a->PopInt();
-  m_test_left                = (uint)a->PopInt();
-  m_test_right               = (uint)a->PopInt();
-  m_test_top                 = (uint)a->PopInt();
-  m_test_bottom              = (uint)a->PopInt();
-  m_width                    = (uint)a->PopInt();
-  m_height                   = (uint)a->PopInt();
-  m_alive                    = (alive_t)a->PopInt();
-  m_energy                   = a->PopInt();
-  m_allow_negative_y         = !!a->PopInt();
+  ASSERT(m_width >= 0);
+  ASSERT(m_height >= 0);
+  SetPhysSize( (Double)newSize.x / PIXEL_PER_METER, (Double)newSize.y/PIXEL_PER_METER );
 }
 
 void PhysicalObj::SetOverlappingObject(PhysicalObj* obj, int timeout)
@@ -177,13 +139,13 @@ void PhysicalObj::SetOverlappingObject(PhysicalObj* obj, int timeout)
   m_minimum_overlapse_time = 0;
   if(obj != NULL) {
     m_overlapping_object = obj;
-    lst_objects.AddOverlappedObject(this);
+    ObjectsList::GetRef().AddOverlappedObject(this);
     MSG_DEBUG("physic.overlapping", "\"%s\" doesn't check any collision with \"%s\" anymore during %d ms",
               GetName().c_str(), obj->GetName().c_str(), timeout);
   } else {
     if(m_overlapping_object != NULL) {
       m_overlapping_object = NULL;
-      lst_objects.RemoveOverlappedObject(this);
+      ObjectsList::GetRef().RemoveOverlappedObject(this);
       MSG_DEBUG( "physic.overlapping", "clearing overlapping object in \"%s\"", GetName().c_str());
     }
     return;
@@ -221,6 +183,11 @@ void PhysicalObj::SetTestRect (uint left, uint right, uint top, uint bottom)
   m_test_right = right;
   m_test_top = top;
   m_test_bottom = bottom;
+
+  ASSERT(m_test_left >= 0);
+  ASSERT(m_test_right >= 0);
+  ASSERT(m_test_top >= 0);
+  ASSERT(m_test_bottom >= 0);
 }
 
 void PhysicalObj::SetEnergyDelta(int delta, bool /*do_report*/)
@@ -251,10 +218,13 @@ collision_t PhysicalObj::NotifyMove(Point2d oldPos, Point2d newPos)
   newPos *= PIXEL_PER_METER;
 
   // Compute distance between old and new position.
-  double lg = oldPos.Distance( newPos);
+  Double lg = oldPos.Distance( newPos);
 
-  MSG_DEBUG("physic.move", "%s moves (%f, %f) -> (%f, %f), distance: %f",
-            typeid(*this).name(), oldPos.x, oldPos.y, newPos.x, newPos.y, lg);
+  MSG_DEBUG("physic.move", "%s moves (%s, %s) -> (%s, %s), distance: %s",
+            typeid(*this).name(), 
+            Double2str(oldPos.x).c_str(), Double2str(oldPos.y).c_str(), 
+            Double2str(newPos.x).c_str(), Double2str(newPos.y).c_str(),
+            Double2str(lg).c_str());
 
   if (lg == 0)
     return NO_COLLISION;
@@ -266,11 +236,13 @@ collision_t PhysicalObj::NotifyMove(Point2d oldPos, Point2d newPos)
   // First iteration position.
   pos = oldPos + offset;
 
-  if (m_goes_through_wall || IsInWater())
+  if (!m_collides_with_ground || IsInWater())
   {
-    MSG_DEBUG("physic.move", "%s moves (%f, %f) -> (%f, %f), thr_wall:%d, water:%d",
-              typeid(*this).name(), oldPos.x, oldPos.y, newPos.x, newPos.y,
-              m_goes_through_wall, IsInWater());
+    MSG_DEBUG("physic.move", "%s moves (%s, %s) -> (%s, %s), collides ground:%d, water:%d",
+              typeid(*this).name(), 
+              Double2str(oldPos.x).c_str(), Double2str(oldPos.y).c_str(), 
+              Double2str(newPos.x).c_str(), Double2str(newPos.y).c_str(),
+              m_collides_with_ground, IsInWater());
 
     SetXY(newPos);
     return NO_COLLISION;
@@ -278,14 +250,14 @@ collision_t PhysicalObj::NotifyMove(Point2d oldPos, Point2d newPos)
 
   do
   {
-    Point2i tmpPos( lround(pos.x), lround(pos.y) );
+    Point2i tmpPos(round(pos.x), round(pos.y) );
 
-    // Check if we exit the world. If so, we stop moving and return.
+    // Check if we exit the GetWorld(). If so, we stop moving and return.
     if( IsOutsideWorldXY(tmpPos) ){
 
-      if( !world.IsOpen() ){
-        tmpPos.x = InRange_Long(tmpPos.x, 0, world.GetWidth() - GetWidth() - 1);
-        tmpPos.y = InRange_Long(tmpPos.y, 0, world.GetHeight() - GetHeight() - 1);
+      if( !GetWorld().IsOpen() ){
+        tmpPos.x = InRange_Long(tmpPos.x, 0, GetWorld().GetWidth() - GetWidth() - 1);
+        tmpPos.y = InRange_Long(tmpPos.y, 0, GetWorld().GetHeight() - GetHeight() - 1);
         MSG_DEBUG( "physic.state", "%s - DeplaceTestCollision touche un bord : %d, %d",  m_name.c_str(), tmpPos.x, tmpPos.y );
         collision = COLLISION_ON_GROUND;
         break;
@@ -294,7 +266,9 @@ collision_t PhysicalObj::NotifyMove(Point2d oldPos, Point2d newPos)
       SetXY( pos );
 
       MSG_DEBUG("physic.move", "%s moves (%f, %f) -> (%f, %f) : OUTSIDE WORLD",
-                typeid(*this).name(), oldPos.x, oldPos.y, newPos.x, newPos.y);
+                typeid(*this).name(), 
+                Double2str(oldPos.x).c_str(), Double2str(oldPos.y).c_str(), 
+                Double2str(newPos.x).c_str(), Double2str(newPos.y).c_str());
       return NO_COLLISION;
     }
 
@@ -320,7 +294,7 @@ collision_t PhysicalObj::NotifyMove(Point2d oldPos, Point2d newPos)
     // Next motion step
     pos += offset;
     lg -= 1.0 ;
-  } while (0 < lg);
+  } while (ZERO < lg);
 
   Point2d speed_before_collision = GetSpeed();
   Point2d speed_collided_obj;
@@ -342,13 +316,10 @@ collision_t PhysicalObj::NotifyMove(Point2d oldPos, Point2d newPos)
     break;
   case COLLISION_ON_GROUND:
     SignalGroundCollision(speed_before_collision);
-    SignalCollision(speed_before_collision);
     break;
   case COLLISION_ON_OBJECT:
-    SignalObjectCollision(collided_obj, speed_before_collision);
-    collided_obj->SignalObjectCollision(this, speed_collided_obj);
-    SignalCollision(speed_before_collision);
-    collided_obj->SignalCollision(speed_collided_obj);
+    SignalObjectCollision(speed_before_collision, collided_obj, speed_collided_obj);
+    collided_obj->SignalObjectCollision(speed_collided_obj, this, speed_before_collision);
     break;
   }
   // ===================================
@@ -359,7 +330,7 @@ collision_t PhysicalObj::NotifyMove(Point2d oldPos, Point2d newPos)
 void PhysicalObj::Collide(collision_t collision, PhysicalObj* collided_obj, const Point2d& position)
 {
   Point2d contactPos;
-  double contactAngle;
+  Double contactAngle;
 
   switch (collision) {
   case NO_COLLISION:
@@ -380,7 +351,7 @@ void PhysicalObj::Collide(collision_t collision, PhysicalObj* collided_obj, cons
     // thanks to Rebound()
 
     // Get the current speed
-    double v1, v2, mass1, angle1, angle2, mass2;
+    Double v1, v2, mass1, angle1, angle2, mass2;
     collided_obj->GetSpeed(v1, angle1);
     GetSpeed(v2, angle2);
     mass1 = GetMass();
@@ -398,7 +369,8 @@ void PhysicalObj::Collide(collision_t collision, PhysicalObj* collided_obj, cons
   }
 
   // Make it rebound!!
-  MSG_DEBUG("physic.state", "m_name.c_str() rebounds at %d,%d", m_name.c_str(), contactPos.x, contactPos.y);
+  MSG_DEBUG("physic.state", "m_name.c_str() rebounds at %s,%s", m_name.c_str(), 
+            Double2str(contactPos.x).c_str(), Double2str(contactPos.y).c_str());
 
   Rebound(contactPos, contactAngle);
   CheckRebound();
@@ -406,7 +378,7 @@ void PhysicalObj::Collide(collision_t collision, PhysicalObj* collided_obj, cons
 
 void PhysicalObj::ContactPointAngleOnGround(const Point2d& oldPos,
 					    Point2d& contactPos,
-					    double& contactAngle) const
+					    Double& contactAngle) const
 {
       // Find the contact point and collision angle.
 //       // !!! ContactPoint(...) _can_ return false when CollisionTest(...) is true !!!
@@ -416,10 +388,10 @@ void PhysicalObj::ContactPointAngleOnGround(const Point2d& oldPos,
     int cx, cy;
 
     if (ContactPoint(cx, cy)) {
-      contactAngle = world.ground.Tangent(cx, cy);
+      contactAngle = GetWorld().ground.Tangent(cx, cy);
       if(!isNaN(contactAngle)) {
-        contactPos.x = (double)cx / PIXEL_PER_METER;
-        contactPos.y = (double)cy / PIXEL_PER_METER;
+        contactPos.x = (Double)cx / PIXEL_PER_METER;
+        contactPos.y = (Double)cy / PIXEL_PER_METER;
       } else {
         contactAngle = - GetSpeedAngle();
         contactPos = oldPos;
@@ -433,12 +405,16 @@ void PhysicalObj::ContactPointAngleOnGround(const Point2d& oldPos,
 void PhysicalObj::UpdatePosition ()
 {
   // No ghost allowed here !
-  if (IsGhost()) return;
+  if (IsGhost()) { 
+    return;
+  }
 
-  if ( !m_goes_through_wall )
+  if (m_collides_with_ground)
     {
       // object is not moving and has no reason to move
-      if ( !IsMoving() && !FootsInVacuum() && !IsInWater() ) return;
+      if ( !IsMoving() && !FootsInVacuum() && !IsInWater() ) {
+      return;
+      }
 
       // object is not moving BUT it should fall !
       if ( !IsMoving() && FootsInVacuum() ) StartMoving();
@@ -447,10 +423,12 @@ void PhysicalObj::UpdatePosition ()
   // Compute new position.
   RunPhysicalEngine();
 
-  if (IsGhost()) return;
+  if (IsGhost()) {
+    return;
+  }
 
   // Classical object sometimes sinks in water and sometimes goes out of water!
-  if ( !m_goes_through_wall )
+  if (m_collides_with_ground)
     {
       if ( IsInWater() && m_alive != DROWNED && m_alive != DEAD) Drown();
       else if ( !IsInWater() && m_alive == DROWNED ) GoOutOfWater();
@@ -458,7 +436,7 @@ void PhysicalObj::UpdatePosition ()
 
 }
 
-bool PhysicalObj::PutOutOfGround(double direction, double max_distance)
+bool PhysicalObj::PutOutOfGround(Double direction, Double max_distance)
 {
   if(IsOutsideWorld(Point2i(0, 0)))
     return false;
@@ -466,11 +444,11 @@ bool PhysicalObj::PutOutOfGround(double direction, double max_distance)
   if( IsInVacuum(Point2i(0, 0), false) )
     return true;
 
-  double dx = cos(direction);
-  double dy = sin(direction);
+  Double dx = cos(direction);
+  Double dy = sin(direction);
   // (dx,dy) is a normal vector (cos^2+sin^2==1)
 
-  double step=1;
+  Double step=1;
   while( step<max_distance && !IsInVacuum(
                           Point2i((int)(dx * step),(int)(dy * step)), false ))
     step+=1.0;
@@ -492,10 +470,10 @@ bool PhysicalObj::PutOutOfGround()
     return true;
 
   bool left,right,top,bottom;
-  left   = world.IsInVacuum_left(*this, 0, 0);
-  right  = world.IsInVacuum_right(*this, 0, 0);
-  top    = world.IsInVacuum_top(*this, 0, 0);
-  bottom = world.IsInVacuum_bottom(*this, 0, 0);
+  left   = GetWorld().IsInVacuum_left(*this, 0, 0);
+  right  = GetWorld().IsInVacuum_right(*this, 0, 0);
+  top    = GetWorld().IsInVacuum_top(*this, 0, 0);
+  bottom = GetWorld().IsInVacuum_bottom(*this, 0, 0);
 
   int dx = (int)GetTestRect().GetSizeX() * (right-left);
   int dy = (int)GetTestRect().GetSizeY() * (top-bottom);
@@ -505,7 +483,7 @@ bool PhysicalObj::PutOutOfGround()
 
   Point2i b(dx, dy);
 
-  double dir = b.ComputeAngle();
+  Double dir = b.ComputeAngle();
   return PutOutOfGround(dir);
 }
 
@@ -540,7 +518,7 @@ void PhysicalObj::Drown()
   m_alive = DROWNED;
 
   // Set the air grab to water resist factor.
-  SetAirResistFactor(WATER_RESIST_FACTOR * GetAirResistFactor());
+  SetAirResistFactor(m_cfg.m_water_resist_factor);
 
   // Ensure the gravity factor is upper than 0.0
   if (EqualsZero(GetGravityFactor()))
@@ -548,10 +526,10 @@ void PhysicalObj::Drown()
 
   // If fire, do smoke...
   if (m_is_fire)
-    world.water.Smoke(GetPosition());
+    GetWorld().water.Smoke(GetPosition());
   // make a splash in the water :-)
   else if (GetMass() >= 2)
-    world.water.Splash(GetPosition());
+    GetWorld().water.Splash(GetPosition());
 
   StopMoving();
   StartMoving();
@@ -575,29 +553,34 @@ void PhysicalObj::SignalRebound()
 {
   // TO CLEAN...
    if (!m_rebound_sound.empty())
-     JukeBox::GetInstance()->Play("share", m_rebound_sound) ;
+     JukeBox::GetInstance()->Play("default", m_rebound_sound) ;
 }
 
-void PhysicalObj::SetCollisionModel(bool goes_through_wall,
+void PhysicalObj::SetCollisionModel(bool collides_with_ground,
                                     bool collides_with_characters,
                                     bool collides_with_objects)
 {
-  m_goes_through_wall = goes_through_wall;
+  m_collides_with_ground = collides_with_ground;
   m_collides_with_characters = collides_with_characters;
   m_collides_with_objects = collides_with_objects;
 
   // Check boolean values
   {
     if (m_collides_with_characters || m_collides_with_objects)
-      ASSERT(m_goes_through_wall == false);
+      ASSERT(m_collides_with_ground);
 
-    if (m_goes_through_wall) {
-      ASSERT(m_collides_with_characters == false);
-      ASSERT(m_collides_with_objects == false);
+    if (!m_collides_with_ground) {
+      ASSERT(!m_collides_with_characters);
+      ASSERT(!m_collides_with_objects);
     }
   }
 }
 
+void PhysicalObj::CanBeGhost(bool state) 
+{
+  can_be_ghost = state;
+}
+
 void PhysicalObj::CheckRebound()
 {
   // If we bounce twice in a row at the same place, stop bouncing
@@ -619,9 +602,9 @@ bool PhysicalObj::IsOutsideWorldXY(const Point2i& position) const{
   int x = position.x + m_test_left;
   int y = position.y + m_test_top;
 
-  if( world.IsOutsideWorldXwidth(x, GetTestWidth()) )
+  if( GetWorld().IsOutsideWorldXwidth(x, GetTestWidth()) )
           return true;
-  if( world.IsOutsideWorldYheight(y, GetTestHeight()) ){
+  if( GetWorld().IsOutsideWorldYheight(y, GetTestHeight()) ){
     if( m_allow_negative_y )
       if( (Y_OBJET_MIN <= y) && (y + GetTestHeight() - 1 < 0) )
                   return false;
@@ -630,22 +613,10 @@ bool PhysicalObj::IsOutsideWorldXY(const Point2i& position) const{
   return false;
 }
 
-bool PhysicalObj::FootsOnFloor(int y) const
-{
-  // If outside is empty, the object can't hit the ground !
-  if ( world.IsOpen() ) return false;
-
-  const int y_max = world.GetHeight()-m_height +m_test_bottom;
-  return (y_max <= y);
-}
-
 bool PhysicalObj::IsInVacuumXY(const Point2i &position, bool check_object) const
 {
   if( IsOutsideWorldXY(position) )
-    return world.IsOpen();
-
-  if( FootsOnFloor(position.y - 1) )
-    return false;
+    return GetWorld().IsOpen();
 
   if( check_object && CollidedObjectXY(position) )
     return false;
@@ -657,7 +628,7 @@ bool PhysicalObj::IsInVacuumXY(const Point2i &position, bool check_object) const
   Rectanglei rect(position.x + m_test_left, position.y + m_test_top,
                   width, height);
 
-  return world.RectIsInVacuum (rect);
+  return GetWorld().RectIsInVacuum (rect);
 }
 
 PhysicalObj* PhysicalObj::CollidedObjectXY(const Point2i & position) const
@@ -701,12 +672,7 @@ bool PhysicalObj::FootsInVacuumXY(const Point2i &position) const
 {
   if( IsOutsideWorldXY(position) ){
     MSG_DEBUG("physical", "%s - physobj is outside the world", m_name.c_str());
-    return world.IsOpen();
-  }
-
-  if( FootsOnFloor(position.y) ){
-    MSG_DEBUG("physical", "%s - physobj is on floor", m_name.c_str());
-    return false;
+    return GetWorld().IsOpen();
   }
 
   int y_test = position.y + m_height - m_test_bottom;
@@ -724,21 +690,21 @@ bool PhysicalObj::FootsInVacuumXY(const Point2i &position) const
   if(CollidedObjectXY( position + Point2i(0, 1)) != NULL )
     return false;
 
-  return world.RectIsInVacuum (rect);
+  return GetWorld().RectIsInVacuum (rect);
 }
 
 bool PhysicalObj::IsInWater () const
 {
   ASSERT (!IsGhost());
-  if (!world.water.IsActive()) return false;
-  int x = InRange_Long(GetCenterX(), 0, world.GetWidth()-1);
-  return (int)world.water.GetHeight(x) < GetCenterY();
+  if (!GetWorld().water.IsActive()) return false;
+  int x = InRange_Long(GetCenterX(), 0, GetWorld().GetWidth()-1);
+  return (int)GetWorld().water.GetHeight(x) < GetCenterY();
 }
 
 void PhysicalObj::DirectFall()
 {
   while (!IsGhost() && !IsInWater() && FootsInVacuum())
-    SetY(GetYdouble()+1.0);
+    SetY(GetYDouble()+ONE);
 }
 
 bool PhysicalObj::ContactPoint (int & contact_x, int & contact_y) const
@@ -746,41 +712,51 @@ bool PhysicalObj::ContactPoint (int & contact_x, int & contact_y) const
   int x1, x2, y1, y2;
 
   // We are looking for a point in contact with the bottom of the object:
-  y1 = (GetY()+m_height-m_test_bottom);
-  y2 = y1-1;
-  for (uint x=GetX()+ m_test_left; x<=(GetX()+m_width)-m_test_right; x++)
-  {
-    if(!world.IsOutsideWorld(Point2i(x,y1)) && !world.IsOutsideWorld(Point2i(x,y2))
-    && world.ground.IsEmpty(Point2i(x,y2)) && !world.ground.IsEmpty(Point2i(x,y1)))
-    {
+  y1 = GetY() + m_height - m_test_bottom;
+  y2 = y1 - 1;
+  Point2i pointA;
+  Point2i pointB;
+
+  for (int x = GetX() + m_test_left; x <= GetX() + m_width - m_test_right; x++) {
+    pointA.SetValues(x, y1);
+    pointB.SetValues(x, y2);
+ 
+    if (!GetWorld().IsOutsideWorld(pointA) && 
+        !GetWorld().IsOutsideWorld(pointB) && 
+        GetWorld().ground.IsEmpty(pointB) && 
+        !GetWorld().ground.IsEmpty(pointA)) {
       contact_x = x;
-      contact_y = GetY() +m_height-m_test_bottom;
+      contact_y = GetY() + m_height - m_test_bottom;
       return true;
     }
   }
 
   // We are looking for a point in contact on the left hand of object:
-  x1 = GetX()+m_test_left;
-  x2 = x1+1;
-  for(uint y=GetY()+m_test_top;y<=GetY()+m_height-m_test_bottom;y++)
-  {
-    if(!world.IsOutsideWorld(Point2i(x1,y)) && !world.IsOutsideWorld(Point2i(x2,y))
-    && !world.ground.IsEmpty(Point2i(x1,y)) &&  world.ground.IsEmpty(Point2i(x2,y)))
-    {
-      contact_x = GetX() +m_test_left;
+  x1 = GetX() + m_test_left;
+  x2 = x1 + 1;
+  
+  for (int y = GetY() + m_test_top; y <= GetY() + m_height - m_test_bottom; y++) {
+    pointA.SetValues(x1, y);
+    pointB.SetValues(x2, y);
+
+    if (!GetWorld().IsOutsideWorld(pointA) && !GetWorld().IsOutsideWorld(pointB) && 
+        !GetWorld().ground.IsEmpty(pointA) &&  GetWorld().ground.IsEmpty(pointB)) {
+      contact_x = GetX() + m_test_left;
       contact_y = y;
       return true;
     }
   }
 
   // We are looking for a point in contact on the rigth hand of object:
-  x1 = (GetX()+m_width-m_test_right);
-  x2 = x1-1;
-  for(uint y=GetY()+m_test_top;y<=GetY()+m_height-m_test_bottom;y++)
-  {
-    if(!world.IsOutsideWorld(Point2i(x1, y)) && !world.IsOutsideWorld(Point2i(x2, y))
-       && !world.ground.IsEmpty(Point2i(x1, y)) && world.ground.IsEmpty(Point2i(x2, y)))
-    {
+  x1 = GetX() + m_width - m_test_right;
+  x2 = x1 - 1;
+
+  for (int y = GetY() + m_test_top; y <= GetY() + m_height - m_test_bottom; y++) {
+    pointA.SetValues(x1, y);
+    pointB.SetValues(x2, y);
+
+    if (!GetWorld().IsOutsideWorld(pointA) && !GetWorld().IsOutsideWorld(pointB) && 
+        !GetWorld().ground.IsEmpty(pointA) && GetWorld().ground.IsEmpty(pointB)) {
       contact_x = GetX() + m_width - m_test_right;
       contact_y = y;
       return true;
@@ -788,22 +764,23 @@ bool PhysicalObj::ContactPoint (int & contact_x, int & contact_y) const
   }
 
   // We are looking for a point in contact on top of object:
-  y1 = GetY()+m_test_top;
+  y1 = GetY() + m_test_top;
   y2 = y1 - 1;
-  for(uint x=GetX()+m_test_left;x<=GetX()+m_width-m_test_right;x++)
-  {
-    if(!world.IsOutsideWorld(Point2i(x,y1)) && !world.IsOutsideWorld(Point2i(x,y2))
-    && !world.ground.IsEmpty(Point2i(x, y1)) && world.ground.IsEmpty(Point2i(x, y2)))
-    {
-      contact_x =x;
-      contact_y = GetY() +m_test_top;
+  for (int x = GetX() + m_test_left; x <= GetX() + m_width - m_test_right; x++) {
+    pointA.SetValues(x, y1);
+    pointB.SetValues(x, y2);
+
+    if (!GetWorld().IsOutsideWorld(pointA) && !GetWorld().IsOutsideWorld(pointB) && 
+        !GetWorld().ground.IsEmpty(pointA) && GetWorld().ground.IsEmpty(pointB)) {
+      contact_x = x;
+      contact_y = GetY() + m_test_top;
       return true;
     }
   }
   return false;
 }
 
-bool PhysicalObj::PutRandomly(bool on_top_of_world, double min_dst_with_characters, bool net_sync)
+bool PhysicalObj::PutRandomly(bool on_top_of_world, Double min_dst_with_characters, bool net_sync)
 {
   uint bcl=0;
   uint NB_MAX_TRY = 60;
@@ -825,30 +802,34 @@ bool PhysicalObj::PutRandomly(bool on_top_of_world, double min_dst_with_characte
 
     if (on_top_of_world) {
       // Give a random position for x
-      if(net_sync)
-        position.x = RandomSync().GetLong(0, world.GetWidth() - GetWidth());
-      else
-        position.x = RandomLocal().GetLong(0, world.GetWidth() - GetWidth());
+      if(net_sync) {
+        MSG_DEBUG("random.get", "PhysicalObj::PutRandomly(...)");
+        position.x = RandomSync().GetLong(0, GetWorld().GetWidth() - GetWidth());
+      } else {
+        position.x = RandomLocal().GetLong(0, GetWorld().GetWidth() - GetWidth());
+      }
       position.y = -GetHeight()+1;
     } else {
-      if(net_sync)
-        position = RandomSync().GetPoint(world.GetSize() - GetSize() + 1);
-      else
-        position = RandomLocal().GetPoint(world.GetSize() - GetSize() + 1);
+      if(net_sync) {
+        MSG_DEBUG("random.get", "PhysicalObj::PutRandomly(...)");
+        position = RandomSync().GetPoint(GetWorld().GetSize() - GetSize() + 1);
+      } else {
+        position = RandomLocal().GetPoint(GetWorld().GetSize() - GetSize() + 1);
+      }
     }
     SetXY(position);
     MSG_DEBUG("physic.position", "%s (try %u/%u) - Test in %d, %d",
               m_name.c_str(), bcl, NB_MAX_TRY, position.x, position.y);
 
     // Check physical object is not in the ground
-    ok &= !IsGhost() && world.ParanoiacRectIsInVacuum(GetTestRect())  && IsInVacuum( Point2i(0, 1) );
+    ok &= !IsGhost() && GetWorld().ParanoiacRectIsInVacuum(GetTestRect())  && IsInVacuum( Point2i(0, 1) );
     if (!ok) {
       MSG_DEBUG("physic.position", "%s - Put it in the ground -> try again !", m_name.c_str());
       continue;
     }
 
     /* check if the area rigth under the object has a bottom on the ground */
-    ok &= !world.ParanoiacRectIsInVacuum(Rectanglei(GetCenter().x, position.y, 1, world.GetHeight() -
+    ok &= !GetWorld().ParanoiacRectIsInVacuum(Rectanglei(GetCenter().x, position.y, 1, GetWorld().GetHeight() -
              (WATER_INITIAL_HEIGHT + 30) - position.y));
     if (!ok) {
       MSG_DEBUG("physic.position", "%s - Put in outside the map or in water -> try again", m_name.c_str());
@@ -860,6 +841,7 @@ bool PhysicalObj::PutRandomly(bool on_top_of_world, double min_dst_with_characte
     // Check distance with characters
     FOR_ALL_LIVING_CHARACTERS(team, character) if (&(*character) != this)
     {
+      MSG_DEBUG("physic.position", "Checking distance to %s", (*character).m_name.c_str(), (*character).m_name.c_str());
       if (min_dst_with_characters == 0) {
 
         if(Overlapse(*character)) {
@@ -869,7 +851,7 @@ bool PhysicalObj::PutRandomly(bool on_top_of_world, double min_dst_with_characte
       } else {
         Point2i p1 = character->GetCenter();
         Point2i p2 = GetCenter();
-        double dst = p1.Distance( p2 );
+        Double dst = p1.Distance( p2 );
 
         // ok this test is not perfect but quite efficient ;-)
         // else we need to check each distance between each "corner"
