@@ -1,6 +1,6 @@
 /******************************************************************************
  *  Wormux is a convivial mass murder game.
- *  Copyright (C) 2001-2008 Wormux Team.
+ *  Copyright (C) 2001-2010 Wormux Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,8 +28,8 @@
 #define PHYSICAL_OBJECT_H
 
 #include "physics.h"
-#include "tool/point.h"
-#include "tool/rectangle.h"
+#include <WORMUX_point.h>
+#include <WORMUX_rectangle.h>
 
 // Alive state
 typedef enum
@@ -42,9 +42,9 @@ typedef enum
 
 class Action;
 
-extern const double PIXEL_PER_METER;
+extern const Double PIXEL_PER_METER;
 
-double MeterDistance (const Point2i &p1, const Point2i &p2);
+Double MeterDistance (const Point2i &p1, const Point2i &p2);
 
 class PhysicalObj : public Physics
 {
@@ -54,11 +54,19 @@ class PhysicalObj : public Physics
 
 private:
   // collision management
-  bool m_goes_through_wall;
+  bool m_collides_with_ground;
   bool m_collides_with_characters;
   bool m_collides_with_objects;
   Point2i m_rebound_position;
 
+  // Rectangle used for collision tests
+  int m_test_left, m_test_right, m_test_top, m_test_bottom;
+
+  // Object size and position.
+  int m_width, m_height;
+
+  bool can_be_ghost;
+
 protected:
   PhysicalObj* m_overlapping_object;
   uint m_minimum_overlapse_time;
@@ -71,12 +79,6 @@ protected:
   std::string m_name;
   std::string m_unique_id;
 
-  // Rectangle used for collision tests
-  uint m_test_left, m_test_right, m_test_top, m_test_bottom;
-
-  // Object size and position.
-  uint m_width, m_height;
-
   std::string m_rebound_sound;
 
   alive_t m_alive;
@@ -94,20 +96,18 @@ public:
 
   //-------- Set position and size -------
 
+  void CanBeGhost(bool state);
+
   // Set/Get position
-  void SetX(double x) { SetXY( Point2d(x, GetYdouble()) ); };
-  void SetY(double y) { SetXY( Point2d(GetXdouble(), y) ); };
+  void SetX(Double x) { SetXY( Point2d(x, GetYDouble()) ); };
+  void SetY(Double y) { SetXY( Point2d(GetXDouble(), y) ); };
   void SetXY(const Point2i &position);
   void SetXY(const Point2d &position);
   int GetX() const;
   int GetY() const;
-  double GetXdouble() const;
-  double GetYdouble() const;
-  const Point2d GetPosition() const { return Point2d(GetXdouble(), GetYdouble()) ;};
-
-  // Used to sync value across network
-  virtual void GetValueFromAction(Action *);
-  virtual void StoreValue(Action *);
+  Double GetXDouble() const;
+  Double GetYDouble() const;
+  const Point2d GetPosition() const { return Point2d(GetXDouble(), GetYDouble()) ;};
 
   // Set/Get size
   void SetSize(const Point2i &newSize);
@@ -138,7 +138,7 @@ public:
   int GetCenterY() const { return GetY() +m_test_top +GetTestHeight()/2; };
   const Point2i GetCenter() const { return Point2i(GetCenterX(), GetCenterY()); };
   const Rectanglei GetRect() const { return Rectanglei( GetX(), GetY(), m_width, m_height); };
-  bool GoesThroughWall() const { return m_goes_through_wall; }
+  bool CollidesWithGround() const { return m_collides_with_ground; }
   bool IsCharacter() const { return m_is_character; }
 
   //----------- Physics related function ----------
@@ -148,12 +148,12 @@ public:
 
   // Move the character until he gets out of the ground
   bool PutOutOfGround();
-  bool PutOutOfGround(double direction, double max_distance=30); //Where direction is the angle of the direction
+  bool PutOutOfGround(Double direction, Double max_distance=30); //Where direction is the angle of the direction
                                          // where the object is moved
                                          // and max_distance is max distance allowed when putting out
 
   // Collision management
-  void SetCollisionModel(bool goes_through_wall,
+  void SetCollisionModel(bool collides_with_ground,
                          bool collides_with_characters,
                          bool collides_with_objects);
   void SetOverlappingObject(PhysicalObj* obj, int timeout = 0);
@@ -204,15 +204,16 @@ public:
   // Do the point p touch the object ?
   bool Contain(const Point2i &p) const { return  GetTestRect().Contains( p ); };
 
-  bool PutRandomly(bool on_top_of_world, double min_dst_with_characters, bool net_sync = true);
+  bool PutRandomly(bool on_top_of_world, Double min_dst_with_characters, bool net_sync = true);
 
   collision_t NotifyMove(Point2d oldPos, Point2d newPos);
 
 protected:
   virtual void SignalRebound();
-  virtual void SignalObjectCollision(PhysicalObj *, const Point2d& /* my_speed_before */) { };
   virtual void SignalGroundCollision(const Point2d& /* my_speed_before */) { };
-  virtual void SignalCollision(const Point2d& /* my_speed_before */) { };
+  virtual void SignalObjectCollision(const Point2d& /* my_speed_before */,
+				     PhysicalObj * /* collided/ing object */,
+				     const Point2d& /* object speed_before */) { };
   virtual void SignalOutOfMap() { };
 
 private:
@@ -230,7 +231,7 @@ private:
 
   void ContactPointAngleOnGround(const Point2d& oldPos,
 				 Point2d& contactPos,
-				 double& contactAngle) const;
+				 Double& contactAngle) const;
 };
 
 #endif
