@@ -1,6 +1,6 @@
 /******************************************************************************
  *  Wormux is a convivial mass murder game.
- *  Copyright (C) 2001-2008 Wormux Team.
+ *  Copyright (C) 2001-2010 Wormux Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
  * Add a structure to the ground
  *****************************************************************************/
 
+#include <WORMUX_types.h>
 #include "weapon/construct.h"
 #include "weapon/explosion.h"
 #include "weapon/weapon_cfg.h"
@@ -32,12 +33,12 @@
 #include "interface/mouse.h"
 #include "map/camera.h"
 #include "map/map.h"
+#include "network/network.h"
 #include "object/objects_list.h"
 #include "sound/jukebox.h"
 #include "team/macro.h"
 #include "team/team.h"
 #include "team/teams_list.h"
-#include "tool/i18n.h"
 #include "tool/resource_manager.h"
 
 #ifdef DEBUG
@@ -46,19 +47,20 @@
 #include "include/app.h"
 #endif
 
-const double DELTA_ANGLE = M_PI / 6.0; // should be a multiple
+const Double DELTA_ANGLE = PI / SIX; // should be a multiple
 
 
 Construct::Construct() : Weapon(WEAPON_CONSTRUCT, "construct",
                                 new WeaponConfig(),
-                                NEVER_VISIBLE)
+                                false)
 {
   UpdateTranslationStrings();
 
-  construct_spr = resource_manager.LoadSprite( weapons_res_profile, "construct_spr");
-  construct_spr->EnableRotationCache(static_cast<int>(2 * M_PI / DELTA_ANGLE));
+  construct_spr = GetResourceManager().LoadSprite( weapons_res_profile, "construct_spr");
+  construct_spr->EnableRotationCache(static_cast<int>(TWO * PI / DELTA_ANGLE));
   m_name = _("Construct");
   m_category = TOOL;
+  m_can_change_weapon = true;
   angle = 0;
   target_chosen = false;
 }
@@ -88,8 +90,8 @@ bool Construct::p_Shoot ()
 {
   if(!target_chosen)
     return false;
-  JukeBox::GetInstance()->Play("share", "weapon/construct");
-  world.MergeSprite(dst - construct_spr->GetSizeMax()/2, construct_spr);
+  JukeBox::GetInstance()->Play("default", "weapon/construct");
+  GetWorld().MergeSprite(dst - construct_spr->GetSizeMax()/2, construct_spr);
 
   target_chosen = false; // ensure next shoot cannot be done pressing key space
   return true;
@@ -97,37 +99,42 @@ bool Construct::p_Shoot ()
 
 void Construct::Draw()
 {
-  if (!IsInUse()) {
-    Weapon::Draw();
+  Weapon::Draw();
 
-    if (EnoughAmmo()
-	&& EnoughAmmoUnit()
-	&& !Interface::GetInstance()->weapons_menu.IsDisplayed()
-	&& Interface::GetInstance()->IsDisplayed()) {
-      dst = Mouse::GetInstance()->GetWorldPosition();
-      construct_spr->SetRotation_rad(angle);
-      construct_spr->Draw(dst - construct_spr->GetSize() / 2);
+  if (EnoughAmmo()
+    && EnoughAmmoUnit()
+    && !Interface::GetInstance()->weapons_menu.IsDisplayed()
+    && Interface::GetInstance()->IsDisplayed()
+    && Network::GetInstance()->IsTurnMaster()) {
+    dst = Mouse::GetInstance()->GetWorldPosition();
+    construct_spr->SetRotation_rad(angle);
+    construct_spr->Draw(dst - construct_spr->GetSize() / 2);
 
 #ifdef DEBUG
-      if (IsLOGGING("test_rectangle"))
-	{
-	  Rectanglei test_rect(dst - construct_spr->GetSizeMax() / 2, construct_spr->GetSizeMax());
-	  test_rect.SetPosition(test_rect.GetPosition() - Camera::GetInstance()->GetPosition());
-	  GetMainWindow().RectangleColor(test_rect, primary_red_color, 1);
-	}
-#endif
+    if (IsLOGGING("test_rectangle")) {
+      Rectanglei test_rect(dst - construct_spr->GetSizeMax() / 2, construct_spr->GetSizeMax());
+      test_rect.SetPosition(test_rect.GetPosition() - Camera::GetInstance()->GetPosition());
+      GetMainWindow().RectangleColor(test_rect, primary_red_color, 1);
     }
+#endif
   }
 }
 
 void Construct::ChooseTarget(Point2i mouse_pos)
 {
+  if (!EnoughAmmo())
+    return;
+
   dst = mouse_pos;
 
+  // Draw it so that GetSizeMax() returns the correct values.
+  construct_spr->SetRotation_rad(angle);
+  construct_spr->Draw(dst - construct_spr->GetSize() / 2);
+
   Point2i test_target = dst - construct_spr->GetSizeMax() / 2;
   Rectanglei rect(test_target, construct_spr->GetSizeMax());
 
-  if (!world.ParanoiacRectIsInVacuum(rect))
+  if (!GetWorld().ParanoiacRectIsInVacuum(rect))
     return;
 
   // Check collision with characters and other physical objects
@@ -148,7 +155,7 @@ void Construct::ChooseTarget(Point2i mouse_pos)
 
 void Construct::Up() const
 {
-  double new_angle = angle + DELTA_ANGLE;
+  Double new_angle = angle + DELTA_ANGLE;
 
   Action* a = new Action(Action::ACTION_WEAPON_CONSTRUCTION, new_angle);
   ActionHandler::GetInstance()->NewAction(a);
@@ -156,7 +163,7 @@ void Construct::Up() const
 
 void Construct::Down() const
 {
-  double new_angle = angle - DELTA_ANGLE;
+  Double new_angle = angle - DELTA_ANGLE;
 
   Action* a = new Action(Action::ACTION_WEAPON_CONSTRUCTION, new_angle);
   ActionHandler::GetInstance()->NewAction(a);
@@ -164,8 +171,3 @@ void Construct::Down() const
 
 WeaponConfig& Construct::cfg()
 { return static_cast<WeaponConfig&>(*extra_params); }
-
-bool Construct::IsInUse() const
-{
-  return m_last_fire_time > 0 && m_last_fire_time + m_time_between_each_shot > Time::GetInstance()->Read();
-}
