@@ -1,6 +1,6 @@
 /******************************************************************************
  *  Wormux is a convivial mass murder game.
- *  Copyright (C) 2001-2008 Wormux Team.
+ *  Copyright (C) 2001-2010 Wormux Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 #include "weapon/grapple.h"
 #include "weapon/weapon_cfg.h"
 
-#include <math.h>
+#include <WORMUX_types.h>
 #include "weapon/explosion.h"
 #include "character/character.h"
 #include "game/config.h"
@@ -32,53 +32,56 @@
 #include "include/action_handler.h"
 #include "map/camera.h"
 #include "map/map.h"
-#include "network/network.h"
 #include "sound/jukebox.h"
 #include "team/team.h"
 #include "team/teams_list.h"
 #include "tool/math_tools.h"
-#include "tool/i18n.h"
 #include "tool/resource_manager.h"
 #include "tool/xml_document.h"
+#include "tool/string_tools.h"
 
-const uint DT_MVT = 15 ; //delta_t between 2 up/down/left/right mvt
-const uint DST_MIN = 80 ;  //dst_minimal between 2 nodes
+const uint DT_MVT = 15; //delta_t between 2 up/down/left/right mvt
+const uint DST_MIN = 4;  //dst_minimal between 2 nodes
+const int SKIP_DST = 8;
 
-bool find_first_contact_point (Point2i from, double angle, uint length,
+bool find_first_contact_point (Point2i from, Double angle, uint length,
                                int skip, Point2i &contact_point)
 {
   Point2d posd;
-  double x_step, y_step ;
+  Double x_step, y_step;
 
-  x_step = cos(angle) ;
-  y_step = sin(angle) ;
+  if ((int)length <= skip) /* to avoid integer overflow */
+    return false;
 
-  posd.x = (double)from.x ;
-  posd.y = (double)from.y ;
+  x_step = cos(angle);
+  y_step = sin(angle);
 
-  posd.x += ((double)skip) * x_step;
-  posd.y += ((double)skip) * y_step;
+  posd.x = (Double)from.x;
+  posd.y = (Double)from.y;
 
-  from.x = (int)round(posd.x) ;
-  from.y = (int)round(posd.y) ;
+  posd.x += ((Double)skip) * x_step;
+  posd.y += ((Double)skip) * y_step;
 
-  contact_point.x = from.x ;
-  contact_point.y = from.y ;
+  from.x = (int)round(posd.x);
+  from.y = (int)round(posd.y);
+
+  contact_point.x = from.x;
+  contact_point.y = from.y;
 
   length -= skip;
 
   // make it return the last point still in vacuum
   Point2i new_contact_point = contact_point;
   bool contact_point_uncertain = true;
-  while(!world.IsOutsideWorld(new_contact_point) &&
+  while(!GetWorld().IsOutsideWorld(new_contact_point) &&
         (length > 0))
     {
-      if ( !world.IsInVacuum( new_contact_point ) )
+      if ( !GetWorld().IsInVacuum( new_contact_point ) )
       {
-        ASSERT( contact_point_uncertain || world.IsInVacuum( contact_point ) );
+        ASSERT( contact_point_uncertain || GetWorld().IsInVacuum( contact_point ) );
 
         // for uncertain contact points, see if it's in vacuum
-        if ( contact_point_uncertain && !world.IsInVacuum( contact_point ) )
+        if ( contact_point_uncertain && !GetWorld().IsInVacuum( contact_point ) )
         {
            // it's not, so try our best to return a contact point in vacuum
            // try searching in area NxN around our original point and return
@@ -97,7 +100,7 @@ bool find_first_contact_point (Point2i from, double angle, uint length,
              for ( int j = -search_radius; j <= search_radius; j ++ )
              {
                cur = contact_point + Point2i( i, j );
-               if ( world.IsInVacuum( cur ) )
+               if ( GetWorld().IsInVacuum( cur ) )
                {
                  // check for new closest
                  int distance = i * i + j * j;
@@ -133,7 +136,6 @@ class GrappleConfig : public EmptyWeaponConfig
 {
  public:
   uint max_rope_length; // Max rope length in pixels
-  uint automatic_growing_speed; // Pixel per 1/100 second.
   int push_force;
 
  public:
@@ -143,22 +145,26 @@ class GrappleConfig : public EmptyWeaponConfig
 
 //-----------------------------------------------------------------------------
 
-Grapple::Grapple() : Weapon(WEAPON_GRAPPLE, "grapple", new GrappleConfig())
+Grapple::Grapple() :
+  Weapon(WEAPON_GRAPPLE, "grapple", new GrappleConfig())
 {
   UpdateTranslationStrings();
 
   m_category = MOVE;
   use_unit_on_first_shoot = false;
 
-  m_hook_sprite = resource_manager.LoadSprite(weapons_res_profile,"grapple_hook");
+  m_hook_sprite = GetResourceManager().LoadSprite(weapons_res_profile,"grapple_hook");
   m_hook_sprite->EnableRotationCache(32);
-  m_node_sprite = resource_manager.LoadSprite(weapons_res_profile,"grapple_node");
+  m_node_sprite = GetResourceManager().LoadSprite(weapons_res_profile,"grapple_node");
 
-  m_is_active = false;
-  m_attaching = false;
-  go_left = false ;
-  go_right = false ;
-  delta_len = 0 ;
+  attached = false;
+  go_left = false;
+  go_right = false;
+  delta_len = 0;
+  move_left_pressed = false;
+  move_right_pressed = false;
+  move_up_pressed = false;
+  move_down_pressed = false;
 }
 
 void Grapple::UpdateTranslationStrings()
@@ -178,12 +184,12 @@ bool Grapple::p_Shoot()
 {
   last_broken_node_angle = 100;
 
-  m_attaching = true;
-  m_launch_time = Time::GetInstance()->Read() ;
-  m_initial_angle = ActiveCharacter().GetFiringAngle();
-  last_mvt=Time::GetInstance()->Read();
+  last_mvt = Time::GetInstance()->Read();
 
-  if (TryAttachRope()) JukeBox::GetInstance()->Play("share", "weapon/grapple_attaching");
+  if (!TryAttachRope()) // We have failed to attach!
+    return false;
+
+  JukeBox::GetInstance()->Play("default", "weapon/grapple_attaching");
   return true;
 }
 
@@ -191,90 +197,50 @@ bool Grapple::TryAttachRope()
 {
   Point2i pos;
   uint length;
-  uint delta_time = Time::GetInstance()->Read() - m_launch_time;
-  double angle ;
+  Double angle;
 
-  // Remove the root node
-  rope_nodes.clear();
+  ASSERT(rope_nodes.empty());
 
-  // The rope is being launching. Increase the rope length and check
-  // collisions.
+  ActiveCharacter().GetHandPosition(pos);
 
-  Point2i handPos = ActiveCharacter().GetHandPosition();
-  pos = handPos;
-
-  length = cfg().automatic_growing_speed * delta_time / 10;
-  if (length > cfg().max_rope_length)
-    {
-      // Hum the rope is too short !
-      m_attaching = false;
-      m_is_active = false;
-
-      // Give back one ammo...
-      int *ammo = &ActiveTeam().AccessNbAmmos();
-      if (*ammo != INFINITE_AMMO) (*ammo)++;
-      ASSERT (*ammo > 0 || *ammo == INFINITE_AMMO);
-
-      return false;
-    }
-
-  angle = m_initial_angle;
+  length = cfg().max_rope_length;
+  angle = ActiveCharacter().GetFiringAngle();
 
   Point2i contact_point;
-  if (find_first_contact_point(pos, angle, length, 4, contact_point))
-    {
-      if (!ActiveTeam().IsLocal() && !ActiveTeam().IsLocalAI())
-        return false;
-
-      // The rope reaches the fixation point. Let's fix it !
-      Action* a = new Action(Action::ACTION_WEAPON_GRAPPLE);
-      a->Push(ATTACH_ROPE);
-      a->Push(contact_point);
-      ActionHandler::GetInstance()->NewAction(a);
-
-      MSG_DEBUG("grapple.hook", "Creating ATTACH_GRAPPLE Action");
-      return true;
-    }
-
-  rope_node_t root_node;
-  root_node.pos.x = pos.x + int(length * cos(angle));
-  root_node.pos.y = pos.y + int(length * sin(angle));
-  root_node.angle = 0;
-  root_node.sense = 0;
-  rope_nodes.push_back(root_node);
+  if (find_first_contact_point(pos, angle, length, SKIP_DST, contact_point)) {
+    AttachRope(contact_point);
+    return true;
+  }
 
   return false;
 }
 
-bool Grapple::TryAddNode(int CurrentSense)
+bool Grapple::TryAddNode()
 {
   uint lg;
   Point2d V;
   Point2i contact_point;
-  double angle, rope_angle;
+  Double angle, rope_angle;
+  Point2i handPos;
 
-  Point2i handPos = ActiveCharacter().GetHandPosition();
+  ActiveCharacter().GetHandPosition(handPos);
 
   // Compute distance between hands and rope fixation point.
 
   V.x = handPos.x - m_fixation_point.x;
   V.y = handPos.y - m_fixation_point.y;
   angle = V.ComputeAngle();
-  lg = static_cast<uint>(V.Norm());
+  lg = static_cast<int>(V.Norm());
 
   if (lg < DST_MIN)
     return false;
 
   // Check if the rope collide something
 
-  if (find_first_contact_point(m_fixation_point, angle, lg, 2, contact_point))
+  if (find_first_contact_point(m_fixation_point, angle, lg, SKIP_DST, contact_point))
     {
       rope_angle = ActiveCharacter().GetRopeAngle() ;
 
-      if ( (last_broken_node_sense * CurrentSense > 0) &&
-           (fabs(last_broken_node_angle - rope_angle) < 0.1))
-        return false ;
-
       // if contact point is the same as position of the last node
       // (can happen because of jitter applied in find_first_contact_point),
       // give up adding such node
@@ -283,15 +249,7 @@ bool Grapple::TryAddNode(int CurrentSense)
 
       // The rope has collided something...
       // Add a node on the rope and change the fixation point
-      AttachNode(contact_point, rope_angle, CurrentSense);
-
-      // Send node addition over the network
-      Action a(Action::ACTION_WEAPON_GRAPPLE);
-      a.Push(ATTACH_NODE);
-      a.Push(contact_point);
-      a.Push(rope_angle);
-      a.Push(CurrentSense);
-      Network::GetInstance()->SendAction(a);
+      AttachNode(contact_point, rope_angle);
 
       return true;
     }
@@ -299,90 +257,46 @@ bool Grapple::TryAddNode(int CurrentSense)
   return false;
 }
 
-bool Grapple::TryRemoveNodes(int currentSense)
+void Grapple::TryRemoveNodes()
 {
-  if ( rope_nodes.size() < 2 )
-    return false;
+  std::list<rope_node_t>::reverse_iterator nodeit;
 
-  // [RCL]: nodeSense check seems to be useless... either remove node senses at all or
-  // find an example where it is required
-  double currentAngle = ActiveCharacter().GetRopeAngle();
-  Point2i mapRopeStart = ActiveCharacter().GetHandPosition();
+  int lg;
+  Point2d V;
+  Point2i handPos, contact_point;
+  Double angle;
 
-  const int max_nodes_per_turn = 100; // safe value to avoid network congestion
-  int nodes_to_remove = 0;
+  ActiveCharacter().GetHandPosition(handPos);
 
-  TraceResult tr;
+  while (rope_nodes.size() > 1) {
 
-  for ( std::list<rope_node_t>::reverse_iterator it = rope_nodes.rbegin();
-       it != rope_nodes.rend(); it++ )
-  {
-    if ( nodes_to_remove >= max_nodes_per_turn )
-        break;
+    nodeit = rope_nodes.rbegin();
+    ++nodeit;
 
-    // try tracing to current node:
-    // if we cannot trace, this means that previous node shouldn't have been removed
-    // (NOTE: since nodes are often in ground, we're ignoring traces hitting ground
-    // right at the end)
-    const float end_proximity_threshold = 0.95f;
-    if ( world.TraceRay( mapRopeStart, it->pos, tr ) && tr.m_fraction < end_proximity_threshold )
-    {
-        // collision detected!
-        if ( nodes_to_remove > 0 )
-            nodes_to_remove--; // undo the node remove
+    V.x = handPos.x - nodeit->pos.x;
+    V.y = handPos.y - nodeit->pos.y;
+    angle = V.ComputeAngle();
+    lg = static_cast<int>(V.Norm());
 
-        // now we can stop removing the nodes as we don't have the clear "sight"
-        // to current node
-        break;
-    };
+    if (find_first_contact_point(nodeit->pos, angle, lg, SKIP_DST, contact_point))
+      break;
 
-    double nodeAngle = it->angle;
-
-    int currentAngleSign = ( currentAngle < 0 ) ? -1 : 1;
-    int nodeAngleSign = ( nodeAngle < 0 ) ? -1 : 1;
-
-    if ( currentAngleSign != nodeAngleSign && rope_nodes.size() > 2 )
-        nodes_to_remove++;
-    else
-        break;
-
-  };
-
-  if ( nodes_to_remove > 0 )
-    MSG_DEBUG( "grapple.break", "nodes to remove %d", nodes_to_remove );
-
-  for ( int i = 0; i < nodes_to_remove; i ++ )
-  {
-     last_broken_node_angle = currentAngle;
-     last_broken_node_sense = currentSense;
-
-     // remove last node
-     DetachNode();
-     // Send node suppression over the network
-     Action a(Action::ACTION_WEAPON_GRAPPLE);
-     a.Push(DETACH_NODE);
-     Network::GetInstance()->SendAction(a);
+    DetachNode();
   }
-
-  return nodes_to_remove > 0;
 }
 
 void Grapple::NotifyMove(bool collision)
 {
   bool addNode = false;
-  int currentSense;
 
-  if (!IsInUse() || m_attaching)
-    return;
-
-  if (!ActiveTeam().IsLocal() && !ActiveTeam().IsLocalAI())
+  if (!attached)
     return;
 
   // Check if the character collide something.
   if (collision)
     {
       // Yes there has been a collision.
-      if (delta_len != 0)
+      if (delta_len != ZERO)
         {
           // The character tryed to change the rope size.
           // There has been a collision, so we cancel the rope length change.
@@ -392,10 +306,9 @@ void Grapple::NotifyMove(bool collision)
       return;
     }
 
-  currentSense = ActiveCharacter().GetAngularSpeed() >= 0 ? 1: -1;
 
   // While there is nodes to add, we add !
-  while (TryAddNode(currentSense))
+  while (TryAddNode())
     addNode = true;
 
   // If we have created nodes, we exit to avoid breaking what we
@@ -403,83 +316,74 @@ void Grapple::NotifyMove(bool collision)
   if (addNode)
     return;
 
-  TryRemoveNodes( currentSense );
+  TryRemoveNodes();
 }
 
 void Grapple::Refresh()
 {
-  if (!IsInUse())
+  if (!attached)
     return ;
 
-  if (m_attaching)
-    TryAttachRope();
-
-  if (!ActiveTeam().IsLocal() && !ActiveTeam().IsLocalAI())
-    return;
-
-if (IsInUse() && !m_attaching)
-  {
-    ActiveCharacter().SetMovement("ninja-rope");
-    ActiveCharacter().UpdatePosition();
-    SendActiveCharacterInfo(true);
+  if (move_left_pressed && !move_right_pressed) {
+    GoLeft();
+  } else if (move_right_pressed && !move_left_pressed) {
+    GoRight();
   }
+
+  if (move_up_pressed && !move_down_pressed) {
+    GoUp();
+  } else if (move_down_pressed && !move_up_pressed) {
+    GoDown();
+  }
+
+  ActiveCharacter().SetMovement("ninja-rope");
+  ActiveCharacter().UpdatePosition();
 }
 
 void Grapple::Draw()
 {
   int x, y;
-  double angle, prev_angle;
+  Double angle, prev_angle;
+  Point2i handPos;
 
   struct CL_Quad {Sint16 x1,x2,x3,x4,y1,y2,y3,y4;} quad;
 
   Weapon::Draw();
 
-  if (!IsInUse())
-  {
-    return ;
-  }
-
-  if (m_attaching) {
-    angle = m_initial_angle + M_PI/2;
-  }
-  else {
-    angle = ActiveCharacter().GetRopeAngle();
+  if (!attached) {
+    return;
   }
 
+  angle = ActiveCharacter().GetRopeAngle();
   prev_angle = angle;
 
   // Draw the rope.
-  Point2i handPos = ActiveCharacter().GetHandPosition();
+  ActiveCharacter().GetHandPosition(handPos);
   x = handPos.x;
   y = handPos.y;
 
-  quad.x1 = (int)round((double)x - 2 * cos(angle));
-  quad.y1 = (int)round((double)y + 2 * sin(angle));
-  quad.x2 = (int)round((double)x + 2 * cos(angle));
-  quad.y2 = (int)round((double)y - 2 * sin(angle));
+  quad.x1 = (int)round((Double)x - 2 * cos(angle));
+  quad.y1 = (int)round((Double)y + 2 * sin(angle));
+  quad.x2 = (int)round((Double)x + 2 * cos(angle));
+  quad.y2 = (int)round((Double)y - 2 * sin(angle));
 
   for (std::list<rope_node_t>::reverse_iterator it = rope_nodes.rbegin();
        it != rope_nodes.rend(); it++)
     {
-      quad.x3 = (int)round((double)it->pos.x + 2 * cos(angle));
-      quad.y3 = (int)round((double)it->pos.y - 2 * sin(angle));
-      quad.x4 = (int)round((double)it->pos.x - 2 * cos(angle));
-      quad.y4 = (int)round((double)it->pos.y + 2 * sin(angle));
+      quad.x3 = (int)round((Double)it->pos.x + 2 * cos(angle));
+      quad.y3 = (int)round((Double)it->pos.y - 2 * sin(angle));
+      quad.x4 = (int)round((Double)it->pos.x - 2 * cos(angle));
+      quad.y4 = (int)round((Double)it->pos.y + 2 * sin(angle));
 
-      float dx = sin(angle) * (float)m_node_sprite->GetHeight();
-      float dy = cos(angle) * (float)m_node_sprite->GetHeight();
+      Double dx = sin(angle) * (Double)m_node_sprite->GetHeight();
+      Double dy = cos(angle) * (Double)m_node_sprite->GetHeight();
       int step = 0;
       int size = (quad.x1-quad.x4) * (quad.x1-quad.x4)
                 +(quad.y1-quad.y4) * (quad.y1-quad.y4);
       size -= m_node_sprite->GetHeight();
-      while( (step*dx*step*dx)+(step*dy*step*dy) < size )
-      {
-        if(m_attaching)
-          m_node_sprite->Draw(Point2i(quad.x1 + (int)((float) step * dx),
-                                      quad.y1 - (int)((float) step * dy)));
-        else
-          m_node_sprite->Draw(Point2i(quad.x4 + (int)((float) step * dx),
-                                      quad.y4 + (int)((float) step * dy)));
+      while( (step*dx*step*dx)+(step*dy*step*dy) < size ) {
+	m_node_sprite->Draw(Point2i(quad.x4 + (int)((Double) step * dx),
+				    quad.y4 + (int)((Double) step * dy)));
         step++;
       }
       quad.x1 = quad.x4 ;
@@ -498,74 +402,68 @@ void Grapple::AttachRope(const Point2i& contact_point)
 {
   MSG_DEBUG("grapple.hook", "** AttachRope %d,%d", contact_point.x, contact_point.y);
 
-  m_attaching = false;
-  m_is_active = true;
+  attached = true;
+  move_left_pressed = false;
+  move_right_pressed = false;
+  move_up_pressed = false;
+  move_down_pressed = false;
 
   rope_nodes.clear();
 
   // The rope reaches the fixation point. Let's fix it !
-  Point2i handPos = ActiveCharacter().GetHandPosition();
-  Point2i pos(handPos.x - ActiveCharacter().GetX(),
-              handPos.y - ActiveCharacter().GetY());
+  Point2i pos;
+  ActiveCharacter().GetRelativeHandPosition(pos);
 
   ActiveCharacter().SetPhysFixationPointXY(
                                            contact_point.x / PIXEL_PER_METER,
                                            contact_point.y / PIXEL_PER_METER,
-                                           (double)pos.x / PIXEL_PER_METER,
-                                           (double)pos.y / PIXEL_PER_METER);
+                                           (Double)pos.x / PIXEL_PER_METER,
+                                           (Double)pos.y / PIXEL_PER_METER);
 
   m_fixation_point = contact_point;
 
   rope_node_t root_node;
   root_node.pos = m_fixation_point;
   root_node.angle = 0;
-  root_node.sense = 0;
   rope_nodes.push_back(root_node);
 
-  ActiveCharacter().ChangePhysRopeSize (-10.0 / PIXEL_PER_METER);
-  m_hooked_time = Time::GetInstance()->Read();
+  ActiveCharacter().ChangePhysRopeSize (((Double)(-10)) / PIXEL_PER_METER);
   ActiveCharacter().SetMovement("ninja-rope");
 
-  ActiveCharacter().SetFiringAngle(-M_PI / 3);
+  ActiveCharacter().SetFiringAngle(-PI / 3);
 
   // Camera should focus on it!
-  Camera::GetInstance()->FollowObject (&ActiveCharacter(), true);
+  Camera::GetInstance()->FollowObject(&ActiveCharacter());
 }
 
 void Grapple::DetachRope()
 {
-  if(m_is_active)
-    ActiveCharacter().UnsetPhysFixationPoint() ;
+  ActiveCharacter().UnsetPhysFixationPoint();
   rope_nodes.clear();
-  m_is_active = false;
+  attached = false;
 
   cable_sound.Stop();
 }
 
-void Grapple::AttachNode(const Point2i& contact_point,
-                         double angle,
-                         int sense)
+void Grapple::AttachNode(const Point2i& contact_point, Double angle)
 {
-  Point2i handPos = ActiveCharacter().GetHandPosition();
-
   // The rope has collided something...
   // Add a node on the rope and change the fixation point.
-  Point2i pos(handPos.x - ActiveCharacter().GetX(),
-              handPos.y - ActiveCharacter().GetY());
+  Point2i pos;
+  ActiveCharacter().GetRelativeHandPosition(pos);
 
   ActiveCharacter().SetPhysFixationPointXY(contact_point.x / PIXEL_PER_METER,
                                            contact_point.y / PIXEL_PER_METER,
-                                           (double)pos.x / PIXEL_PER_METER,
-                                           (double)pos.y / PIXEL_PER_METER);
+                                           (Double)pos.x / PIXEL_PER_METER,
+                                           (Double)pos.y / PIXEL_PER_METER);
 
   m_fixation_point = contact_point;
   rope_node_t node;
   node.pos = m_fixation_point;
   node.angle = angle;
-  node.sense = sense;
   rope_nodes.push_back(node);
 
-  MSG_DEBUG("grapple.node", "+ %d,%d %f %d", node.pos.x, node.pos.y, node.angle, node.sense);
+  MSG_DEBUG("grapple.node", "+ %d,%d %s", node.pos.x, node.pos.y, Double2str(node.angle).c_str());
 }
 
 void Grapple::DetachNode()
@@ -576,7 +474,7 @@ void Grapple::DetachNode()
   { // for debugging only
     rope_node_t node;
     node = rope_nodes.back();
-    MSG_DEBUG("grapple.node", "- %d,%d %f %d", node.pos.x, node.pos.y, node.angle, node.sense);
+    MSG_DEBUG("grapple.node", "- %d,%d %s", node.pos.x, node.pos.y, Double2str(node.angle).c_str());
   }
 #endif
 
@@ -585,21 +483,20 @@ void Grapple::DetachNode()
 
   m_fixation_point = rope_nodes.back().pos ;
 
-  Point2i handPos = ActiveCharacter().GetHandPosition();
-  int dx = handPos.x - ActiveCharacter().GetX();
-  int dy = handPos.y - ActiveCharacter().GetY();
+  Point2i pos;
+  ActiveCharacter().GetRelativeHandPosition(pos);
 
   ActiveCharacter().SetPhysFixationPointXY(m_fixation_point.x / PIXEL_PER_METER,
                                            m_fixation_point.y / PIXEL_PER_METER,
-                                           (double)dx / PIXEL_PER_METER,
-                                           (double)dy / PIXEL_PER_METER);
+                                           (Double)pos.x / PIXEL_PER_METER,
+                                           (Double)pos.y / PIXEL_PER_METER);
 }
 
 // =========================== Moves management
 
-void Grapple::SetRopeSize(double length) const
+void Grapple::SetRopeSize(Double length) const
 {
-  double delta = length - ActiveCharacter().GetRopeLength();
+  Double delta = length - ActiveCharacter().GetRopeLength();
   ActiveCharacter().ChangePhysRopeSize (delta);
 }
 
@@ -643,7 +540,7 @@ void Grapple::StopDown()
 void Grapple::GoRight()
 {
   if (!go_right) {
-    cable_sound.Play("share", "weapon/grapple_cable");
+    cable_sound.Play("default", "weapon/grapple_cable");
   }
   go_right = true ;
   ActiveCharacter().SetExternForce(cfg().push_force,0);
@@ -664,7 +561,7 @@ void Grapple::StopRight()
 void Grapple::GoLeft()
 {
   if (!go_left) {
-    cable_sound.Play("share", "weapon/grapple_cable");
+    cable_sound.Play("default", "weapon/grapple_cable");
   }
   go_left = true ;
   ActiveCharacter().SetExternForce(-cfg().push_force,0);
@@ -682,124 +579,134 @@ void Grapple::StopLeft()
   ActiveCharacter().SetExternForce(0,0);
 }
 
+void Grapple::StartMovingLeft()
+{
+  move_left_pressed = true;
+}
+
+void Grapple::StopMovingLeft()
+{
+  move_left_pressed = false;
+  StopLeft();
+}
+
+void Grapple::StartMovingRight()
+{
+  move_right_pressed = true;
+}
+
+void Grapple::StopMovingRight()
+{
+  move_right_pressed = false;
+  StopRight();
+}
+
+void Grapple::StartMovingUp()
+{
+  move_up_pressed = true;
+}
+
+void Grapple::StopMovingUp()
+{
+  move_up_pressed = false;
+  StopUp();
+}
+
+void Grapple::StartMovingDown()
+{
+  move_down_pressed = true;
+  cable_sound.Play("default", "weapon/grapple_cable", -1);
+}
+
+void Grapple::StopMovingDown()
+{
+  move_down_pressed = false;
+  StopDown();
+}
+
+bool Grapple::IsPreventingLRMovement()
+{
+  return attached;
+}
+
+bool Grapple::IsPreventingWeaponAngleChanges()
+{
+  return attached;
+}
+
 // =========================== Keys management
 
-void Grapple::HandleKeyPressed_Up(bool shift)
+void Grapple::HandleKeyPressed_Up(bool /*slowly*/)
 {
-  if (IsInUse())  {
-    cable_sound.Play("share", "weapon/grapple_cable", -1);
-    GoUp();
-  }
+  if (attached)
+    StartMovingUpForAllPlayers();
+}
+
+void Grapple::HandleKeyReleased_Up(bool /*slowly*/)
+{
+  if (attached)
+    StopMovingUpForAllPlayers();
+}
+
+void Grapple::HandleKeyPressed_Down(bool /*slowly*/)
+{
+  if (attached)
+    StartMovingDownForAllPlayers();
+}
+
+void Grapple::HandleKeyReleased_Down(bool /*slowly*/)
+{
+  if (attached)
+    StopMovingDownForAllPlayers();
+}
+
+void Grapple::HandleKeyPressed_MoveLeft(bool /*slowly*/)
+{
+  if (attached)
+    StartMovingLeftForAllPlayers();
+}
+
+void Grapple::HandleKeyReleased_MoveLeft(bool /*slowly*/)
+{
+  if (attached)
+    StopMovingLeftForAllPlayers();
+}
+
+void Grapple::HandleKeyPressed_MoveRight(bool /*slowly*/)
+{
+  if (attached)
+    StartMovingRightForAllPlayers();
+}
+
+void Grapple::HandleKeyReleased_MoveRight(bool /*slowly*/)
+{
+  if (attached)
+    StopMovingRightForAllPlayers();
+}
+
+void Grapple::StartShooting()
+{
+  if (!attached)
+    Weapon::StartShooting();
+}
+
+void Grapple::StopShooting()
+{
+  if (attached)
+    DetachRope();
   else
-    ActiveCharacter().HandleKeyPressed_Up(shift);
-}
-
-void Grapple::HandleKeyRefreshed_Up(bool shift)
-{
-  if (IsInUse())
-    GoUp();
-  else
-    ActiveCharacter().HandleKeyRefreshed_Up(shift);
-}
-
-void Grapple::HandleKeyReleased_Up(bool shift)
-{
-  if (IsInUse())
-    StopUp();
-  else
-    ActiveCharacter().HandleKeyReleased_Up(shift);
-}
-
-void Grapple::HandleKeyPressed_Down(bool shift)
-{
-  if (IsInUse()) {
-    cable_sound.Play("share", "weapon/grapple_cable", -1);
-    GoDown();
-  } else
-    ActiveCharacter().HandleKeyPressed_Down(shift);
-}
-
-void Grapple::HandleKeyRefreshed_Down(bool shift)
-{
-  if (IsInUse())
-    GoDown();
-  else
-    ActiveCharacter().HandleKeyRefreshed_Down(shift);
-}
-
-void Grapple::HandleKeyReleased_Down(bool shift)
-{
-  if (IsInUse())
-    StopDown();
-  else
-    ActiveCharacter().HandleKeyReleased_Down(shift);
-}
-
-void Grapple::HandleKeyPressed_MoveLeft(bool shift)
-{
-  if (IsInUse())
-    GoLeft();
-  else
-    ActiveCharacter().HandleKeyPressed_MoveLeft(shift);
-}
-
-void Grapple::HandleKeyRefreshed_MoveLeft(bool shift)
-{
-  if (!IsInUse())
-    ActiveCharacter().HandleKeyRefreshed_MoveLeft(shift);
-}
-
-void Grapple::HandleKeyReleased_MoveLeft(bool shift)
-{
-  if (IsInUse())
-    StopLeft();
-  else
-    ActiveCharacter().HandleKeyReleased_MoveLeft(shift);
-}
-
-void Grapple::HandleKeyPressed_MoveRight(bool shift)
-{
-  if (IsInUse())
-    GoRight();
-  else
-    ActiveCharacter().HandleKeyPressed_MoveRight(shift);
-}
-
-void Grapple::HandleKeyRefreshed_MoveRight(bool shift)
-{
-  if (!IsInUse())
-    ActiveCharacter().HandleKeyRefreshed_MoveRight(shift);
-}
-
-void Grapple::HandleKeyReleased_MoveRight(bool shift)
-{
-  if (IsInUse())
-    StopRight();
-  else
-    ActiveCharacter().HandleKeyReleased_MoveRight(shift);
-}
-
-void Grapple::HandleKeyPressed_Shoot(bool)
-{
-  if (IsInUse()) {
-    NewActionWeaponStopUse();
-  } else
-    NewActionWeaponShoot();
+    Weapon::StopShooting();
 }
 
 void Grapple::PrintDebugRope()
 {
-  printf("%05d %05d %03.3f\n",
-         ActiveCharacter().GetX(),
-         ActiveCharacter().GetY(),
-         ActiveCharacter().GetRopeAngle());
+  std::cout << ActiveCharacter().GetX() << " " << ActiveCharacter().GetY() << " " << ActiveCharacter().GetRopeAngle() << std::endl;
 
   for (std::list<rope_node_t>::iterator it = rope_nodes.begin();
        it != rope_nodes.end();
        it++) {
 
-    printf("%05d %05d %03.3f %d\n", it->pos.x, it->pos.y,
-           it->angle, it->sense);
+    std::cout << it->pos.x << " " << it->pos.y << " " << it->angle << std::endl;
   }
 }
 
@@ -822,7 +729,6 @@ GrappleConfig& Grapple::cfg()
 GrappleConfig::GrappleConfig()
 {
   max_rope_length = 450;
-  automatic_growing_speed = 12;
   push_force = 10;
 }
 
@@ -830,6 +736,5 @@ void GrappleConfig::LoadXml(const xmlNode* elem)
 {
   EmptyWeaponConfig::LoadXml(elem);
   XmlReader::ReadUint(elem, "max_rope_length", max_rope_length);
-  XmlReader::ReadUint(elem, "automatic_growing_speed", automatic_growing_speed);
   XmlReader::ReadInt(elem, "push_force", push_force);
 }
