@@ -1,6 +1,6 @@
 /******************************************************************************
  *  Wormux is a convivial mass murder game.
- *  Copyright (C) 2001-2008 Wormux Team.
+ *  Copyright (C) 2001-2010 Wormux Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,12 +19,10 @@
  * WeaponLauncher: generic weapon to launch a projectile
  *****************************************************************************/
 
-#include "weapon/weapon_launcher.h"
-#include "weapon/weapon_cfg.h"
-
 #include <sstream>
+#include <WORMUX_debug.h>
+#include <limits.h>
 
-#include "weapon/explosion.h"
 #include "character/character.h"
 #include "game/config.h"
 #include "game/game.h"
@@ -38,10 +36,12 @@
 #include "team/macro.h"
 #include "team/team.h"
 #include "team/teams_list.h"
-#include "tool/debug.h"
-#include "tool/i18n.h"
 #include "tool/math_tools.h"
 #include "tool/resource_manager.h"
+#include "tool/string_tools.h"
+#include "weapon/explosion.h"
+#include "weapon/weapon_cfg.h"
+#include "weapon/weapon_launcher.h"
 
 #ifdef DEBUG
 //#define DEBUG_EXPLOSION_CONFIG
@@ -49,6 +49,8 @@
 #include "include/app.h"
 #endif
 
+const uint INVALID_TIMEOUT_START = UINT_MAX;
+
 WeaponBullet::WeaponBullet(const std::string &name,
                            ExplosiveWeaponConfig& cfg,
                            WeaponLauncher * p_launcher) :
@@ -62,7 +64,7 @@ WeaponBullet::WeaponBullet(const std::string &name,
 // Signal that the bullet has hit the ground
 void WeaponBullet::SignalGroundCollision(const Point2d& speed_before)
 {
-  JukeBox::GetInstance()->Play("share", "weapon/ricoche1");
+  JukeBox::GetInstance()->Play("default", "weapon/ricoche1");
   WeaponProjectile::SignalGroundCollision(speed_before);
   launcher->IncMissedShots();
 }
@@ -71,32 +73,33 @@ void WeaponBullet::SignalOutOfMap()
 {
   WeaponProjectile::SignalOutOfMap();
   launcher->IncMissedShots();
-  Camera::GetInstance()->FollowObject(&ActiveCharacter(), true);
+  Camera::GetInstance()->FollowObject(&ActiveCharacter());
 }
 
-void WeaponBullet::SignalObjectCollision(PhysicalObj * obj, const Point2d& my_speed_before)
+void WeaponBullet::SignalObjectCollision(const Point2d& my_speed_before,
+					 PhysicalObj * obj,
+					 const Point2d& /*obj_speed*/)
 {
 #if 1
-  if (!obj->IsCharacter())
-    Explosion();
-  obj->SetEnergyDelta(-(int)cfg.damage);
   obj->AddSpeed(cfg.speed_on_hit, my_speed_before.ComputeAngle());
-  Ghost();
 #else
   // multiply by ten to get something more funny
-  double bullet_mass = GetMass()/* * 10*/;
-  double total_mass = bullet_mass + obj->GetMass();
+  Double bullet_mass = GetMass()/* * 10*/;
+  Double total_mass = bullet_mass + obj->GetMass();
   // computing new speed of character
   Point2d v2 = (my_speed_before * (1 + 0.8) * bullet_mass +
                 obj->GetSpeed() * (obj->GetMass() - 0.8 * bullet_mass)) / total_mass;
   // Pushing a little upward character to allow him to be pushed by the projectile
   obj->SetXY(Point2i(obj->GetX(), obj->GetY() - 3));
   obj->SetSpeedXY(v2);
+#endif
+
   obj->SetEnergyDelta(-(int)cfg.damage);
   if (!obj->IsCharacter())
     Explosion();
   Ghost();
-#endif
+
+  WeaponProjectile::Collision();
 }
 
 void WeaponBullet::Refresh()
@@ -108,27 +111,31 @@ void WeaponBullet::Refresh()
 void WeaponBullet::DoExplosion()
 {
   Point2i pos = GetCenter();
-  ApplyExplosion(pos, cfg, "", false, ParticleEngine::LittleESmoke, GetUniqueId());
+  ApplyExplosion(pos, cfg, "", false, ParticleEngine::LittleESmoke);
 }
 //-----------------------------------------------------------------------------
 
 
 WeaponProjectile::WeaponProjectile(const std::string &name,
                                     ExplosiveWeaponConfig& p_cfg,
-                                    WeaponLauncher * p_launcher)
-  : PhysicalObj(name), cfg(p_cfg)
+                                    WeaponLauncher * p_launcher):
+  PhysicalObj(name),
+  timeout_start(INVALID_TIMEOUT_START),
+  cfg(p_cfg)
 {
   m_allow_negative_y = true;
-  SetCollisionModel(false, true, true);
+  SetCollisionModel(true, true, true);
   launcher = p_launcher;
 
   explode_colliding_character = false;
-  explode_with_timeout = true;
+  // explode_with_timeout defaults to false as
+  // the default value true has caused a few hard to find bugs.
+  explode_with_timeout = false;
   explode_with_collision = true;
   can_drown = true;
-  camera_in_advance = true;
+  camera_follow_closely = false;
 
-  image = resource_manager.LoadSprite( weapons_res_profile, name);
+  image = GetResourceManager().LoadSprite( weapons_res_profile, name);
   image->EnableRotationCache(32);
   SetSize(image->GetSize());
 
@@ -148,9 +155,9 @@ WeaponProjectile::~WeaponProjectile()
   delete image;
 }
 
-void WeaponProjectile::Shoot(double strength)
+void WeaponProjectile::Shoot(Double strength)
 {
-  MSG_DEBUG("weapon.projectile", "shoot with strength:%f", strength);
+  MSG_DEBUG("weapon.projectile", "shoot with strength:%s", Double2str(strength).c_str());
 
   Init();
 
@@ -162,34 +169,42 @@ void WeaponProjectile::Shoot(double strength)
 
   // Set the initial position.
   SetOverlappingObject(&ActiveCharacter(), 100);
-  lst_objects.AddObject(this);
-  Camera::GetInstance()->FollowObject(this, true, camera_in_advance);
+  ObjectsList::GetRef().AddObject(this);
 
-  double angle = ActiveCharacter().GetFiringAngle();
+  Double angle = ActiveCharacter().GetFiringAngle();
   RandomizeShoot(angle, strength);
 
+  Point2i hand_position;
+  ActiveCharacter().GetHandPosition(hand_position);
+  ActiveCharacter().GetHandPosition(hand_position);
   MSG_DEBUG("weapon.projectile", "shoot from position %d,%d (size %d, %d) - hand position:%d,%d",
-	    ActiveCharacter().GetX(),
-	    ActiveCharacter().GetY(),
-	    ActiveCharacter().GetWidth(),
-	    ActiveCharacter().GetHeight(),
-            ActiveCharacter().GetHandPosition().GetX(),
-            ActiveCharacter().GetHandPosition().GetY());
-  MSG_DEBUG("weapon.projectile", "shoot with strength:%f, angle:%f, position:%d,%d",
-            strength, angle, GetX(), GetY());
+            ActiveCharacter().GetX(),
+            ActiveCharacter().GetY(),
+            ActiveCharacter().GetWidth(),
+            ActiveCharacter().GetHeight(),
+            hand_position.GetX(),
+            hand_position.GetY());
 
-  begin_time = Time::GetInstance()->Read();
+  MSG_DEBUG("weapon.projectile", "shoot with strength:%s, angle:%s, position:%d,%d",
+            Double2str(strength).c_str(), Double2str(angle).c_str(), GetX(), GetY());
+
+  StartTimeout();
 
   ShootSound();
 
   // bug #10236 : problem with flamethrower collision detection
   // Check if the object is colliding something between hand position and gun hole
-  Point2i hand_position = ActiveCharacter().GetHandPosition() - GetSize() / 2;
+  hand_position -= GetSize() / 2;
   Point2i hole_position = launcher->GetGunHolePosition() - GetSize() / 2;
   Point2d f_hand_position(hand_position.GetX() / PIXEL_PER_METER, hand_position.GetY() / PIXEL_PER_METER);
   Point2d f_hole_position(hole_position.GetX() / PIXEL_PER_METER, hole_position.GetY() / PIXEL_PER_METER);
   SetXY(hand_position);
   SetSpeed(strength, angle);
+
+  // Camera::FollowObject must be called after setting initial speed else
+  // camera_follow_closely will have no effect
+  Camera::GetInstance()->FollowObject(this, camera_follow_closely);
+
   collision_t collision = NotifyMove(f_hand_position, f_hole_position);
   if (collision == NO_COLLISION) {
     // Set the initial position and speed.
@@ -210,11 +225,16 @@ void WeaponProjectile::Refresh()
     Explosion();
     return;
   }
+  // The RefreshSurface() call is necessary as the image determines the size of the projectile
+  // and we don't want to rely on Draw to call RefreshSurface() as the draw rate differs
+  // for the players in a multiplayer game.
+  image->RefreshSurface();
   SetSize(image->GetSizeMax());
-  // Explose after timeout
-  int tmp = Time::GetInstance()->Read() - begin_time;
 
-  if(cfg.timeout && tmp > 1000 * (GetTotalTimeout())) SignalTimeout();
+  if (cfg.timeout) {
+     if (((int)GetMSSinceTimeoutStart()) > 1000 * GetTotalTimeout())
+       SignalTimeout();
+  }
 }
 
 void WeaponProjectile::SetEnergyDelta(int /*delta*/, bool /*do_report*/)
@@ -231,15 +251,15 @@ void WeaponProjectile::Draw()
 
   if (cfg.timeout && tmp != 0)
   {
-    tmp -= (int)((Time::GetInstance()->Read() - begin_time) / 1000);
+    tmp -= (int)((GetMSSinceTimeoutStart()) / 1000);
     if (tmp >= 0)
     {
       std::ostringstream ss;
       ss << tmp ;
       int txt_x = GetX() + GetWidth() / 2;
       int txt_y = GetY() - GetHeight();
-      (*Font::GetInstance(Font::FONT_SMALL)).WriteCenterTop( Point2i(txt_x, txt_y) - Camera::GetInstance()->GetPosition(),
-      ss.str(), white_color);
+	  Text text(ss.str());
+      text.DrawCenterTop(Point2i(txt_x, txt_y) - Camera::GetInstance()->GetPosition());
     }
   }
 
@@ -258,19 +278,23 @@ void WeaponProjectile::Draw()
 
 bool WeaponProjectile::IsImmobile() const
 {
-  if(explode_with_timeout && begin_time + GetTotalTimeout() * 1000 > Time::GetInstance()->Read())
+  if (explode_with_timeout && GetTotalTimeout() * 1000 > (int) GetMSSinceTimeoutStart())
     return false;
   return PhysicalObj::IsImmobile();
 }
 
 // projectile explode and signal to the launcher the collision
-void WeaponProjectile::SignalObjectCollision(PhysicalObj * obj, const Point2d& /* my_speed_before */)
+void WeaponProjectile::SignalObjectCollision(const Point2d& /* my_speed_before */,
+					     PhysicalObj * obj,
+					     const Point2d& /* obj_speed_before */)
 {
   ASSERT(obj != NULL);
   MSG_DEBUG("weapon.projectile", "SignalObjectCollision \"%s\" with \"%s\": %d, %d",
 	    m_name.c_str(), obj->GetName().c_str(), GetX(), GetY());
   if (explode_colliding_character)
     Explosion();
+
+  Collision();
 }
 
 // projectile explode when hiting the ground
@@ -279,16 +303,19 @@ void WeaponProjectile::SignalGroundCollision(const Point2d& /*speed_before*/)
   MSG_DEBUG("weapon.projectile", "SignalGroundCollision \"%s\": %d, %d", m_name.c_str(), GetX(), GetY());
   if (explode_with_collision)
     Explosion();
+
+  Collision();
 }
 
-// Default behavior : signal to launcher a collision and explode
-void WeaponProjectile::SignalCollision(const Point2d& speed_before)
+// Signal to launcher a collision
+void WeaponProjectile::Collision()
 {
-  MSG_DEBUG("weapon.projectile", "SignalCollision \"%s\": %d, %d", m_name.c_str(), GetX(), GetY());
+  MSG_DEBUG("weapon.projectile", "Collision \"%s\": %d, %d", m_name.c_str(), GetX(), GetY());
   if (launcher != NULL && !launcher->ignore_collision_signal)
-    launcher->SignalProjectileCollision(speed_before);
+    launcher->SignalProjectileCollision();
 }
 
+
 // Default behavior : signal to launcher projectile is drowning
 void WeaponProjectile::SignalDrowning()
 {
@@ -297,8 +324,8 @@ void WeaponProjectile::SignalDrowning()
   if (launcher != NULL && !launcher->ignore_drowning_signal)
     launcher->SignalProjectileDrowning();
 
-  if (can_drown) JukeBox::GetInstance()->Play("share", "sink");
-  //else JukeBox::GetInstance()->Play("share", "pschiiit");
+  if (can_drown) JukeBox::GetInstance()->Play("default", "sink");
+  //else JukeBox::GetInstance()->Play("default", "pschiiit");
 }
 
 // Default behavior : signal to launcher a projectile is going out of water
@@ -342,7 +369,7 @@ void WeaponProjectile::SignalExplosion()
 void WeaponProjectile::DoExplosion()
 {
   Point2i pos = GetCenter();
-  ApplyExplosion(pos, cfg, "weapon/explosion", true, ParticleEngine::BigESmoke, GetUniqueId());
+  ApplyExplosion(pos, cfg, "weapon/explosion", true, ParticleEngine::BigESmoke);
 }
 
 void WeaponProjectile::IncrementTimeOut()
@@ -369,6 +396,19 @@ int WeaponProjectile::GetTotalTimeout() const
   return (int)(cfg.timeout)+m_timeout_modifier;
 }
 
+void WeaponProjectile::StartTimeout()
+{
+  timeout_start = Time::GetInstance()->Read();
+}
+
+uint WeaponProjectile::GetMSSinceTimeoutStart() const
+{
+  uint now = Time::GetInstance()->Read();
+  ASSERT (timeout_start  != INVALID_TIMEOUT_START);
+  ASSERT(now >= timeout_start);
+  return now - timeout_start;
+}
+
 // Signal a projectile timeout and explode
 void WeaponProjectile::SignalTimeout()
 {
@@ -390,8 +430,8 @@ bool WeaponProjectile::change_timeout_allowed() const
 WeaponLauncher::WeaponLauncher(Weapon_type type,
                                const std::string &id,
                                EmptyWeaponConfig * params,
-                               weapon_visibility_t visibility) :
-    Weapon(type, id, params, visibility)
+                               bool drawable) :
+    Weapon(type, id, params, drawable)
 {
   projectile = NULL;
   nb_active_projectile = 0;
@@ -411,6 +451,21 @@ WeaponLauncher::~WeaponLauncher()
     delete projectile;
 }
 
+int WeaponLauncher::GetDamage()
+{
+  return cfg().damage;
+}
+
+Double WeaponLauncher::GetWindFactor()
+{
+  return projectile->GetWindFactor();
+}
+
+Double WeaponLauncher::GetMass()
+{
+  return projectile->GetMass();
+}
+
 bool WeaponLauncher::p_Shoot()
 {
 //   if (m_strength == max_strength)
@@ -425,11 +480,6 @@ bool WeaponLauncher::p_Shoot()
   return true;
 }
 
-bool WeaponLauncher::IsInUse() const
-{
-  return m_last_fire_time > 0 && m_last_fire_time + m_time_between_each_shot > Time::GetInstance()->Read();
-}
-
 bool WeaponLauncher::ReloadLauncher()
 {
   if (projectile)
@@ -450,7 +500,7 @@ void WeaponLauncher::Draw()
   //Display timeout for projectil if can be changed.
   if (projectile->change_timeout_allowed())
   {
-    if( IsInUse() ) //Do not display after launching.
+    if (IsOnCooldownFromShot()) //Do not display after launching.
       return;
 
     int tmp = projectile->GetTotalTimeout();
@@ -458,9 +508,9 @@ void WeaponLauncher::Draw()
     ss << tmp;
     ss << "s";
     int txt_x = ActiveCharacter().GetX() + ActiveCharacter().GetWidth() / 2;
-    int txt_y = ActiveCharacter().GetY() - ActiveCharacter().GetHeight();
-    (*Font::GetInstance(Font::FONT_SMALL)).WriteCenterTop( Point2i(txt_x, txt_y) - Camera::GetInstance()->GetPosition(),
-    ss.str(), white_color);
+    int txt_y = ActiveCharacter().GetY() - 4*ActiveCharacter().GetHeight()/5;
+	Text text(ss.str());
+      text.DrawCenterTop(Point2i(txt_x, txt_y) - Camera::GetInstance()->GetPosition());
   }
 
   Weapon::Draw();
@@ -496,88 +546,85 @@ void WeaponLauncher::IncMissedShots()
     GameMessages::GetInstance()->Add (_("Your shot has missed!"));
 }
 
-void WeaponLauncher::HandleKeyReleased_Num1(bool /*shift*/)
+void WeaponLauncher::HandleKeyReleased_Num1()
 {
-  projectile->SetTimeOut(1);
-  NetworkSetTimeoutProjectile();
+  SetTimeoutForAllPlayers(1);
 }
 
-void WeaponLauncher::HandleKeyReleased_Num2(bool /*shift*/)
+void WeaponLauncher::HandleKeyReleased_Num2()
 {
-  projectile->SetTimeOut(2);
-  NetworkSetTimeoutProjectile();
+  SetTimeoutForAllPlayers(2);
 }
 
-void WeaponLauncher::HandleKeyReleased_Num3(bool /*shift*/)
+void WeaponLauncher::HandleKeyReleased_Num3()
 {
-  projectile->SetTimeOut(3);
-  NetworkSetTimeoutProjectile();
+  SetTimeoutForAllPlayers(3);
 }
 
-void WeaponLauncher::HandleKeyReleased_Num4(bool /*shift*/)
+void WeaponLauncher::HandleKeyReleased_Num4()
 {
-  projectile->SetTimeOut(4);
-  NetworkSetTimeoutProjectile();
+  SetTimeoutForAllPlayers(4);
 }
 
-void WeaponLauncher::HandleKeyReleased_Num5(bool /*shift*/)
+void WeaponLauncher::HandleKeyReleased_Num5()
 {
-  projectile->SetTimeOut(5);
-  NetworkSetTimeoutProjectile();
+  SetTimeoutForAllPlayers(5);
 }
 
-void WeaponLauncher::HandleKeyReleased_Num6(bool /*shift*/)
+void WeaponLauncher::HandleKeyReleased_Num6()
 {
-  projectile->SetTimeOut(6);
-  NetworkSetTimeoutProjectile();
+  SetTimeoutForAllPlayers(6);
 }
 
-void WeaponLauncher::HandleKeyReleased_Num7(bool /*shift*/)
+void WeaponLauncher::HandleKeyReleased_Num7()
 {
-  projectile->SetTimeOut(7);
-  NetworkSetTimeoutProjectile();
+  SetTimeoutForAllPlayers(7);
 }
 
-void WeaponLauncher::HandleKeyReleased_Num8(bool /*shift*/)
+void WeaponLauncher::HandleKeyReleased_Num8()
 {
-  projectile->SetTimeOut(8);
-  NetworkSetTimeoutProjectile();
+  SetTimeoutForAllPlayers(8);
 }
 
-void WeaponLauncher::HandleKeyReleased_Num9(bool /*shift*/)
+void WeaponLauncher::HandleKeyReleased_Num9()
 {
-  projectile->SetTimeOut(9);
-  NetworkSetTimeoutProjectile();
+  SetTimeoutForAllPlayers(9);
 }
 
-void WeaponLauncher::HandleKeyReleased_Less(bool /*shift*/)
+void WeaponLauncher::HandleKeyReleased_Less()
 {
-  projectile->DecrementTimeOut();
-  NetworkSetTimeoutProjectile();
+  SetTimeoutForAllPlayers(GetTimeout() - 1);
 }
 
-void WeaponLauncher::HandleKeyReleased_More(bool /*shift*/)
+void WeaponLauncher::HandleKeyReleased_More()
 {
-  projectile->IncrementTimeOut();
-  NetworkSetTimeoutProjectile();
+  SetTimeoutForAllPlayers(GetTimeout() + 1);
 }
 
-void WeaponLauncher::NetworkSetTimeoutProjectile() const
+void WeaponLauncher::SetTimeoutForAllPlayers(int timeout)
 {
-  ActionHandler::GetInstance()->NewAction(new Action(Action::ACTION_WEAPON_SET_TIMEOUT,
-                                                     projectile->m_timeout_modifier));
+  Action * a = new Action(Action::ACTION_WEAPON_SET_TIMEOUT, timeout);
+  ActionHandler::GetInstance()->NewAction(a);
+}
+
+void WeaponLauncher::SetTimeout(int timeout)
+{
+  GetProjectile()->SetTimeOut(timeout);
+}
+
+int WeaponLauncher::GetTimeout()
+{
+  return GetProjectile()->GetTotalTimeout();
 }
 
 void WeaponLauncher::HandleMouseWheelUp(bool /*shift*/)
 {
-  projectile->IncrementTimeOut();
-  NetworkSetTimeoutProjectile();
+  SetTimeoutForAllPlayers(GetTimeout() + 1);
 }
 
 void WeaponLauncher::HandleMouseWheelDown(bool /*shift*/)
 {
-  projectile->DecrementTimeOut();
-  NetworkSetTimeoutProjectile();
+  SetTimeoutForAllPlayers(GetTimeout() - 1);
 }
 
 ExplosiveWeaponConfig& WeaponLauncher::cfg()
